Package: BayesianDisaggregation
Title: Evidence-Based Bayesian Disaggregation of Aggregate Indices
Version: 0.2.1
Depends: R (>= 4.1.0)
Authors@R: person("José Mauricio", "Gómez Julián", email = "isadore.nabi@pm.me", role = c("aut", "cre"), comment = c(ORCID = "0009-0000-2412-3150"))
Description: Disaggregates an observed aggregate price index into sectoral
    components with a Bayesian state-space model in which the aggregate enters as
    a genuine observation density rather than as a renormalization identity. A
    random-walk-with-drift transition in log space (with partial pooling on the
    drift and the innovation scale) and an estimable cross-sectional
    concentration produce posterior draws of the sectoral indices with credible
    intervals, suitable as multiple-imputation input for downstream dynamic
    models. The Hamiltonian Monte Carlo engine follows Stan (Carpenter et al.,
    2017) <doi:10.18637/jss.v076.i01>; model comparison uses Pareto Smoothed
    Importance Sampling Leave-One-Out cross-validation (Vehtari, Gelman and
    Gabry, 2017) <doi:10.1007/s11222-016-9696-4>. A closed-form linear-Gaussian
    Kalman/RTS smoother provides an exact, MCMC-free Bayesian alternative for the
    same aggregate evidence.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: readxl, dplyr, tidyr, stringr, magrittr, stats, parallel
Suggests: cmdstanr, rstan (>= 2.21.0), posterior, loo (>= 2.5.0),
        knitr, rmarkdown, ggplot2, readr, testthat (>= 3.0.0)
Additional_repositories: https://mc-stan.org/r-packages/
VignetteBuilder: knitr
NeedsCompilation: no
Author: José Mauricio Gómez Julián [aut, cre] (ORCID:
    <https://orcid.org/0009-0000-2412-3150>)
Maintainer: José Mauricio Gómez Julián <isadore.nabi@pm.me>
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-18 19:27:15 UTC; josemgomezj
Repository: CRAN
Date/Publication: 2026-06-18 22:10:07 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,disagg_conjugate)
S3method(print,disagg_statespace)
S3method(print,summary.disagg_statespace)
S3method(summary,disagg_statespace)
export(align_disagg_inputs)
export(disagg_default_priors)
export(disagg_stan_code)
export(disaggregate_conjugate)
export(disaggregate_from_files)
export(disaggregate_statespace)
export(log_enable)
export(read_cpi)
export(read_weights_matrix)
export(simulate_disagg)
importFrom(magrittr,"%>%")

# Example Data Files for BayesianDisaggregation

This directory contains example data files for testing and demonstration:

## CPI.xlsx
- Annual Consumer Price Index data (2019-2023)
- Contains aggregate index (Total) and component indices
- Categories: Food, Housing, Transport, Healthcare, Education, Recreation, Other
- Base year: 2019 (index = 100)

## WEIGHTS.xlsx
- Industry weights matrix for CPI components
- Rows: Industries/Categories
- Columns: Years (2019-2023)
- Each year's weights sum to 1.0
- Format: Industry | 2019 | 2020 | 2021 | 2022 | 2023

These are minimal example files for package testing and documentation.
For real analysis, users should provide their own data files with appropriate structure.

## ----setup, include=TRUE------------------------------------------------------
# Valores predeterminados globales de los chunks
knitr::opts_chunk$set(
  echo = TRUE, message = FALSE, warning = FALSE,
  fig.width = 9, fig.height = 6
)

# Librerías
suppressPackageStartupMessages({
  library(BayesianDisaggregation)
  library(dplyr)
  library(tidyr)
  library(ggplot2)
  library(readr)
  library(openxlsx)
})

# Verbosidad del registro desde el paquete
log_enable("INFO")
set.seed(2024)

## ----L_from_P, include=TRUE---------------------------------------------------
# Llamada de ejemplo (los internos están en el paquete):
# L <- compute_L_from_P(P) 

## ----spread-L, include=TRUE---------------------------------------------------
# Llamada de ejemplo:
# LT <- spread_likelihood(L, T_periods = nrow(P), pattern = "recent")

## ----posteriors, include=TRUE-------------------------------------------------
# posterior_weighted(P, LT, lambda = 0.7)
# posterior_multiplicative(P, LT)
# posterior_dirichlet(P, LT, gamma = 0.1)
# posterior_adaptive(P, LT)

## ----metrics-fns, include=TRUE------------------------------------------------
# coherence_score(P, W, L, mult = 3.0, const = 0.5)
# numerical_stability_exp(W, a = 1000, b = 10)
# temporal_stability(W, kappa = 50)
# stability_composite(W, a = 1000, b = 10, kappa = 50)

## ----interp-fn, include=TRUE--------------------------------------------------
# interpretability_score(P, W, use_q90 = TRUE)

## ----api, include=TRUE--------------------------------------------------------
# Firma de ejemplo (ver Sección 8 para datos reales):
# bayesian_disaggregate(path_cpi, path_weights,
#   method = c("weighted","multiplicative","dirichlet","adaptive"),
#   lambda = 0.7, gamma = 0.1,
#   coh_mult = 3.0, coh_const = 0.5,
#   stab_a = 1000, stab_b = 10, stab_kappa = 50,
#   likelihood_pattern = "recent")

## ----demo, include=TRUE-------------------------------------------------------
# Matriz previa sintética (filas en el símplice)
T <- 10; K <- 6
set.seed(123)
P <- matrix(rexp(T*K), nrow = T)
P <- P / rowSums(P)

# Vector de verosimilitud desde P (ACP/SVD; robusto con alternativa)
L  <- compute_L_from_P(P)

# Difundir en el tiempo con patrón "recent"
LT <- spread_likelihood(L, T_periods = T, pattern = "recent")

# Probar un par de posteriores
W_weighted <- posterior_weighted(P, LT, lambda = 0.7)
W_adaptive <- posterior_adaptive(P, LT)

# Métricas para el adaptativo
coh  <- coherence_score(P, W_adaptive, L)
stab <- stability_composite(W_adaptive, a = 1000, b = 10, kappa = 50)
intr <- interpretability_score(P, W_adaptive)
eff  <- 0.65
comp <- 0.30*coh + 0.25*stab + 0.25*intr + 0.20*eff

data.frame(coherence = coh, stability = stab, interpretability = intr,
           efficiency = eff, composite = comp) %>% round(4)

## ----real-pipeline, eval=FALSE------------------------------------------------
# # === Crear datos sintéticos para demo compatible con CRAN ===
# demo_dir <- tempdir()
# 
# # Crear datos sintéticos de IPC (imitando tu estructura)
# set.seed(123)
# cpi_demo <- data.frame(
#   Year = 2000:2010,
#   CPI = cumsum(c(100, rnorm(10, 0.5, 2)))  # Caminata aleatoria iniciando en 100
# )
# cpi_file <- file.path(demo_dir, "synthetic_cpi.xlsx")
# openxlsx::write.xlsx(cpi_demo, cpi_file)
# 
# # Crear matriz de pesos sintética (imitando estructura de pesos de VAB)
# set.seed(456)
# years <- 2000:2010
# sectors <- c("Agriculture", "Manufacturing", "Services", "Construction", "Mining")
# 
# weights_demo <- data.frame(Year = years)
# for(sector in sectors) {
#   weights_demo[[sector]] <- runif(length(years), 0.05, 0.35)
# }
# # Normalizar filas para sumar 1 (restricción de símplice)
# weights_demo[, -1] <- weights_demo[, -1] / rowSums(weights_demo[, -1])
# weights_file <- file.path(demo_dir, "synthetic_weights.xlsx")
# openxlsx::write.xlsx(weights_demo, weights_file)
# 
# # Usar rutas de datos sintéticos
# path_cpi <- cpi_file
# path_w <- weights_file
# out_dir <- demo_dir
# 
# cat("Usando datos sintéticos para la demo de CRAN:\n")
# cat("Archivo CPI:", path_cpi, "\n")
# cat("Archivo de pesos:", path_w, "\n")
# cat("Directorio de salida:", out_dir, "\n")
# 
# # --- Ejecución base (predeterminados robustos) ---
# base_res <- bayesian_disaggregate(
#   path_cpi           = path_cpi,
#   path_weights       = path_w,
#   method             = "adaptive",
#   lambda             = 0.7,   # registrado en métricas; no usado por "adaptive"
#   gamma              = 0.1,
#   coh_mult           = 3.0,
#   coh_const          = 0.5,
#   stab_a             = 1000,
#   stab_b             = 10,
#   stab_kappa         = 60,
#   likelihood_pattern = "recent"
# )
# xlsx_base <- save_results(base_res, out_dir = file.path(out_dir, "base"))
# print(base_res$metrics)
# 
# # --- Búsqueda de cuadrícula mínima para la demo (tamaño reducido) ---
# n_cores <- 1  # Un solo núcleo para cumplimiento CRAN
# grid_df <- expand.grid(
#   method             = c("weighted", "adaptive"),  # Métodos reducidos
#   lambda             = c(0.5, 0.7),               # Opciones reducidas
#   gamma              = 0.1,                       # Opción única
#   coh_mult           = 3.0,                       # Opción única
#   coh_const          = 0.5,                       # Opción única
#   stab_a             = 1000,
#   stab_b             = 10,
#   stab_kappa         = 60,                        # Opción única
#   likelihood_pattern = "recent",                  # Opción única
#   KEEP.OUT.ATTRS     = FALSE,
#   stringsAsFactors   = FALSE
# )
# 
# grid_res <- run_grid_search(
#   path_cpi     = path_cpi,
#   path_weights = path_w,
#   grid_df      = grid_df,
#   n_cores      = n_cores
# )
# write.csv(grid_res, file.path(out_dir, "grid_results.csv"), row.names = FALSE)
# 
# best_row <- grid_res %>% arrange(desc(composite)) %>% slice(1)
# print("Mejor configuración de la búsqueda en cuadrícula:")
# print(best_row)
# 
# # --- Re-ejecutar la mejor configuración para exportación limpia ---
# best_res <- bayesian_disaggregate(
#   path_cpi           = path_cpi,
#   path_weights       = path_w,
#   method             = best_row$method,
#   lambda             = if (!is.na(best_row$lambda)) best_row$lambda else 0.7,
#   gamma              = if (!is.na(best_row$gamma))  best_row$gamma  else 0.1,
#   coh_mult           = best_row$coh_mult,
#   coh_const          = best_row$coh_const,
#   stab_a             = best_row$stab_a,
#   stab_b             = best_row$stab_b,
#   stab_kappa         = best_row$stab_kappa,
#   likelihood_pattern = best_row$likelihood_pattern
# )
# xlsx_best <- save_results(best_res, out_dir = file.path(out_dir, "best"))
# 
# # --- Un Excel con todo (incluyendo hiperparámetros) ---
# sector_summary <- tibble(
#   Sector          = colnames(best_res$posterior)[-1],
#   prior_mean      = colMeans(as.matrix(best_res$prior[, -1])),
#   posterior_mean  = colMeans(as.matrix(best_res$posterior[, -1]))
# )
# 
# wb <- createWorkbook()
# addWorksheet(wb, "Hyperparameters"); writeData(wb, "Hyperparameters", best_row)
# addWorksheet(wb, "Metrics");         writeData(wb, "Metrics", best_res$metrics)
# addWorksheet(wb, "Prior_P");         writeData(wb, "Prior_P", best_res$prior)
# addWorksheet(wb, "Posterior_W");     writeData(wb, "Posterior_W", best_res$posterior)
# addWorksheet(wb, "Likelihood_t");    writeData(wb, "Likelihood_t", best_res$likelihood_t)
# addWorksheet(wb, "Likelihood_L");    writeData(wb, "Likelihood_L", best_res$likelihood)
# addWorksheet(wb, "Sector_Summary");  writeData(wb, "Sector_Summary", sector_summary)
# 
# for (sh in c("Hyperparameters","Metrics","Prior_P","Posterior_W",
#              "Likelihood_t","Likelihood_L","Sector_Summary")) {
#   freezePane(wb, sh, firstRow = TRUE)
#   addFilter(wb, sh, rows = 1, cols = 1:ncol(readWorkbook(wb, sh)))
#   setColWidths(wb, sh, cols = 1:200, widths = "auto")
# }
# 
# # --- Añadir IPC sectorial (agregado por pesos posteriores) ---
# W_post <- best_res$posterior           # Year + sectores
# cpi_df <- read_cpi(path_cpi)           # Year, CPI
# sector_cpi <- dplyr::left_join(W_post, cpi_df, by = "Year") %>%
#   dplyr::mutate(dplyr::across(-c(Year, CPI), ~ .x * CPI))
# 
# # Verificación de calidad: suma de sectores vs CPI
# check_sum <- sector_cpi %>%
#   dplyr::mutate(row_sum = rowSums(dplyr::across(-c(Year, CPI))),
#                 diff    = CPI - row_sum)
# cat("Verificación de calidad (primeras 5 filas):\n")
# print(head(check_sum, 5))
# 
# addWorksheet(wb, "Sector_CPI")
# writeData(wb, "Sector_CPI", sector_cpi)
# freezePane(wb, "Sector_CPI", firstRow = TRUE)
# addFilter(wb, "Sector_CPI", rows = 1, cols = 1:ncol(sector_cpi))
# setColWidths(wb, "Sector_CPI", cols = 1:200, widths = "auto")
# 
# excel_onefile <- file.path(out_dir, "best", "Best_Full_Output_withSectorCPI.xlsx")
# saveWorkbook(wb, excel_onefile, overwrite = TRUE)
# cat("Resultados completos guardados en:", excel_onefile, "\n")
# 
# # --- Gráficos rápidos (guardados como PNGs) ---
# dir_plots <- file.path(out_dir, "best", "plots")
# if (!dir.exists(dir_plots)) dir.create(dir_plots, recursive = TRUE)
# 
# W_long <- best_res$posterior %>%
#   pivot_longer(-Year, names_to = "Sector", values_to = "Weight")
# p_heat <- ggplot(W_long, aes(Year, Sector, fill = Weight)) +
#   geom_tile() + scale_fill_viridis_c() +
#   labs(title = "Pesos posteriores (W): mapa de calor", x = "Año", y = "Sector", fill = "Participación") +
#   theme_minimal(base_size = 11) + theme(axis.text.y = element_text(size = 6))
# ggsave(file.path(dir_plots, "posterior_heatmap.png"), p_heat, width = 12, height = 9, dpi = 220)
# 
# top_sectors <- best_res$posterior %>%
#   summarise(across(-Year, mean)) %>%
#   pivot_longer(everything(), names_to = "Sector", values_to = "MeanShare") %>%
#   arrange(desc(MeanShare)) %>% slice(1:3) %>% pull(Sector)  # Reducido a top 3 para la demo
# 
# p_lines <- best_res$posterior %>%
#   select(Year, all_of(top_sectors)) %>%
#   pivot_longer(-Year, names_to = "Sector", values_to = "Weight") %>%
#   ggplot(aes(Year, Weight, color = Sector)) +
#   geom_line(linewidth = 0.9) +
#   labs(title = "Top 3 sectores por participación media (posterior W)", y = "Participación", x = "Año") +
#   theme_minimal(base_size = 11)
# ggsave(file.path(dir_plots, "posterior_topSectors.png"), p_lines, width = 11, height = 6, dpi = 220)
# 
# cat("Demo completada exitosamente. Todos los archivos escritos al directorio temporal.\n")

## ----invariants, include=TRUE-------------------------------------------------
# Ejemplo: invariantes en una corrida sintética fresca
T <- 6; K <- 5
set.seed(7)
P <- matrix(rexp(T*K), nrow = T); P <- P / rowSums(P)
L <- compute_L_from_P(P)
LT <- spread_likelihood(L, T, "recent")
W  <- posterior_multiplicative(P, LT)

# Invariantes
stopifnot(all(abs(rowSums(P)  - 1) < 1e-12))
stopifnot(all(abs(rowSums(LT) - 1) < 1e-12))
stopifnot(all(abs(rowSums(W)  - 1) < 1e-12))
c(
  coherence = coherence_score(P, W, L),
  stability = stability_composite(W),
  interpret = interpretability_score(P, W)
) %>% round(4)

## ----session, include=TRUE----------------------------------------------------
sessionInfo()

# =============================================================================
# Stan backend plumbing for the canonical (state-space) disaggregation engine.
# Mirrors the conventions used by the sibling package bayesianOU so that both
# behave identically with respect to backend selection, model-file resolution
# and backend-agnostic draw extraction.
# =============================================================================

#' Null-coalescing helper
#' @keywords internal
#' @noRd
`%||%` <- function(x, y) if (is.null(x)) y else x

#' Locate the canonical state-space Stan model file
#'
#' Resolves the path to \code{inst/stan/disagg_statespace.stan} (the single
#' source of truth for the evidence-based engine), both for the installed
#' package and during development (source-tree fallbacks).
#'
#' @return Character path to the \code{.stan} file.
#' @keywords internal
#' @noRd
.disagg_stan_file_path <- function() {
  p <- system.file("stan", "disagg_statespace.stan", package = "BayesianDisaggregation")
  if (nzchar(p) && file.exists(p)) return(p)

  candidates <- c(
    file.path("inst", "stan", "disagg_statespace.stan"),
    file.path("..", "inst", "stan", "disagg_statespace.stan"),
    file.path("..", "..", "inst", "stan", "disagg_statespace.stan")
  )
  for (cand in candidates) if (file.exists(cand)) return(normalizePath(cand))

  stop("Could not locate 'disagg_statespace.stan'. Reinstall the package or run ",
       "from the package root.", call. = FALSE)
}

#' Stan code for the canonical disaggregation model
#'
#' Returns the complete Stan code of the evidence-based state-space model read
#' from the canonical file (single source of truth; no embedded duplicate).
#'
#' @return Character string with the Stan model code.
#' @examples
#' code <- disagg_stan_code()
#' cat(substr(code, 1, 200))
#' @export
disagg_stan_code <- function() {
  paste(readLines(.disagg_stan_file_path(), warn = FALSE), collapse = "\n")
}

#' Detect an available Stan backend (cmdstanr preferred, then rstan)
#' @keywords internal
#' @noRd
check_stan_backend <- function(verbose = FALSE) {
  if (requireNamespace("cmdstanr", quietly = TRUE)) {
    ok <- tryCatch(!is.null(cmdstanr::cmdstan_version(error_on_NA = FALSE)),
                   error = function(e) FALSE)
    if (ok) {
      if (verbose) message("Using cmdstanr backend")
      return("cmdstanr")
    }
  }
  if (requireNamespace("rstan", quietly = TRUE)) {
    if (verbose) message("Using rstan backend")
    return("rstan")
  }
  "none"
}

#' Compile and sample the canonical state-space model (cmdstanr or rstan)
#' @keywords internal
#' @noRd
.run_stan_disagg <- function(stan_dat, chains, iter, warmup, thin, cores,
                             adapt_delta, max_treedepth, seed, init, verbose) {
  backend <- check_stan_backend(verbose)
  if (backend == "none") {
    stop("A Stan backend is required (install 'cmdstanr' or 'rstan').", call. = FALSE)
  }
  par_chains <- max(1L, min(as.integer(chains), as.integer(cores)))

  if (backend == "cmdstanr") {
    if (verbose) message("Compiling state-space model with cmdstanr")
    cache_dir <- tryCatch({
      d <- tools::R_user_dir("BayesianDisaggregation", "cache")
      dir.create(d, recursive = TRUE, showWarnings = FALSE)
      if (dir.exists(d)) d else tempdir()
    }, error = function(e) tempdir())
    mod <- cmdstanr::cmdstan_model(.disagg_stan_file_path(), dir = cache_dir,
                                   pedantic = FALSE)
    if (verbose) message("Running MCMC sampling")
    return(mod$sample(
      data = stan_dat, chains = chains, parallel_chains = par_chains,
      iter_warmup = warmup, iter_sampling = iter - warmup, thin = thin,
      seed = seed, refresh = if (verbose) 200 else 0,
      adapt_delta = adapt_delta, max_treedepth = max_treedepth, init = init
    ))
  }

  if (verbose) message("Compiling state-space model with rstan")
  sm <- rstan::stan_model(model_code = disagg_stan_code())
  rstan_init <- "random"; rstan_init_r <- 2
  if (!is.null(init)) {
    if (is.numeric(init) && length(init) == 1L) rstan_init_r <- init else rstan_init <- init
  }
  rstan::sampling(
    sm, data = stan_dat, chains = chains, iter = iter, warmup = warmup,
    thin = thin, seed = seed,
    control = list(adapt_delta = adapt_delta, max_treedepth = max_treedepth),
    refresh = if (verbose) 200 else 0, init = rstan_init, init_r = rstan_init_r
  )
}

#' Draws as a [draws x columns] matrix for a parameter, backend-agnostic
#' @keywords internal
#' @noRd
.draws_matrix <- function(fit, p) {
  if (inherits(fit, "CmdStanMCMC")) {
    df <- tryCatch(fit$draws(p, format = "df"), error = function(e) NULL)
    if (is.null(df)) return(NULL)
    keep <- grep(sprintf("^%s(\\[|$)", p), names(df), value = TRUE)
    keep <- setdiff(keep, c(".chain", ".iteration", ".draw"))
    if (length(keep) == 0) return(NULL)
    return(as.matrix(df[, keep, drop = FALSE]))
  }
  out <- tryCatch(rstan::extract(fit, pars = p, permuted = TRUE)[[1]],
                  error = function(e) NULL)
  if (is.null(out)) return(NULL)
  if (is.null(dim(out))) return(matrix(out, ncol = 1))
  if (length(dim(out)) == 2L) return(out)
  matrix(out, nrow = dim(out)[1])
}

#' Reshape draws of a T x K matrix parameter into a [T, K, D] array
#'
#' Columns of \code{.draws_matrix(fit, name)} are named \code{name[t,k]} (t
#' fastest, matching cmdstanr/rstan order). Returns a \code{[T, K, D]} array with
#' the imputations on the third margin, the contract consumed by
#' \code{bayesianOU::fit_ou_nested_mi}.
#' @keywords internal
#' @noRd
.matrix_param_to_array <- function(fit, name, Tn, K) {
  M <- .draws_matrix(fit, name)
  if (is.null(M)) return(NULL)
  cn <- colnames(M)
  tt <- as.integer(sub(sprintf("^%s\\[(\\d+),(\\d+)\\]$", name), "\\1", cn))
  kk <- as.integer(sub(sprintf("^%s\\[(\\d+),(\\d+)\\]$", name), "\\2", cn))
  D <- nrow(M)
  arr <- array(NA_real_, dim = c(Tn, K, D))
  for (j in seq_along(cn)) arr[tt[j], kk[j], ] <- M[, j]
  arr
}

#' Backend-agnostic convergence diagnostics (rhat_max, divergences)
#' @keywords internal
#' @noRd
.stan_diagnostics <- function(fit) {
  if (inherits(fit, "CmdStanMCMC")) {
    rhat_max <- tryCatch({
      s <- fit$summary()
      max(s$rhat, na.rm = TRUE)
    }, error = function(e) NA_real_)
    div <- tryCatch({
      dg <- fit$diagnostic_summary(quiet = TRUE)
      sum(dg$num_divergent)
    }, error = function(e) NA_real_)
    return(list(rhat_max = rhat_max, divergences = div))
  }
  rhat_max <- tryCatch(max(rstan::summary(fit)$summary[, "Rhat"], na.rm = TRUE),
                       error = function(e) NA_real_)
  div <- tryCatch({
    sp <- rstan::get_sampler_params(fit, inc_warmup = FALSE)
    sum(vapply(sp, function(x) sum(x[, "divergent__"]), numeric(1)))
  }, error = function(e) NA_real_)
  list(rhat_max = rhat_max, divergences = div)
}

# Golden regression for the state-space computation path.
# Recomputes log_lik via generate_quantities on FROZEN parameter draws + data and
# demands a bit-for-bit match against the frozen reference. Catches any change to
# the Stan model's computed quantities. gq is deterministic (no RNG in the
# generated quantities block), so an exact match is the correct expectation.
# Gated: needs cmdstanr (gq) and the fixture; auto-skips on CRAN.

test_that("state-space generated quantities reproduce the golden log_lik bit-for-bit", {
  skip_on_cran()
  fx <- testthat::test_path("fixtures", "golden_statespace.rds")
  skip_if(!file.exists(fx), "golden fixture absent (run validacion/make_golden_statespace.R)")
  skip_if_not_installed("cmdstanr")
  if (check_stan_backend() != "cmdstanr") skip("golden recompute requires cmdstanr")

  golden <- readRDS(fx)
  mod <- cmdstanr::cmdstan_model(BayesianDisaggregation:::.disagg_stan_file_path(),
                                 dir = tempdir(), pedantic = FALSE)
  gq <- mod$generate_quantities(fitted_params = golden$draws, data = golden$stan_dat)
  ll_new <- gq$draws(variables = "log_lik", format = "draws_matrix")

  expect_equal(dim(ll_new), dim(golden$log_lik_ref))
  expect_identical(as.numeric(ll_new), as.numeric(golden$log_lik_ref))
})

# Recovery of the state-space engine on data simulated from its own DGP.
# Heavy (compiles + samples): gated behind NOT_CRAN and an explicit flag.
#   NOT_CRAN=true BAYESDISAGG_RUN_RECOVERY=1 Rscript -e 'devtools::test(...)'
#
# Rigor by layers: the AGGREGATE is strongly identified (must be recovered
# tightly); the SECTORAL split is weakly identified by construction (the bands
# are honestly wide, so coverage is high and the point correlation is only
# moderate) — we assert the identified quantity tightly and the unidentified one
# conservatively, never claiming sectoral precision that the data cannot give.

test_that("state-space recovers the aggregate and yields the phi_draws contract", {
  skip_on_cran()
  if (Sys.getenv("BAYESDISAGG_RUN_RECOVERY") != "1") skip("heavy recovery gated")
  if (check_stan_backend() == "none") skip("no Stan backend available")

  sim <- simulate_disagg(T = 35, K = 4, seed = 101)
  fit <- disaggregate_statespace(sim$cpi, sim$W, chains = 2, iter = 900, warmup = 450,
                                 seed = 101, adapt_delta = 0.97, max_treedepth = 12)

  # contract consumed by bayesianOU::fit_ou_nested_mi
  expect_equal(dim(fit$phi_draws)[1:2], c(35L, 4L))
  expect_true(dim(fit$phi_draws)[3] >= 100L)
  expect_true(all(is.finite(fit$phi_draws)))

  # convergence
  expect_lt(fit$diagnostics$rhat_max, 1.15)

  # aggregate: strongly identified
  expect_gt(cor(fit$agg_summary[, "median"], sim$cpi), 0.95)
  expect_gt(cor(fit$agg_summary[, "median"], sim$agg_true), 0.95)

  # sectoral: weakly identified -> wide bands -> conservative coverage
  cov95 <- mean(sim$phi_true >= fit$phi_summary$q2.5 & sim$phi_true <= fit$phi_summary$q97.5)
  expect_gt(cov95, 0.70)
})

# =============================================================================
# Closed-form conjugate baseline: linear-Gaussian state-space disaggregation.
#
# The canonical engine (disaggregate_statespace) works on log phi with Student-t
# observations and a partial-pooling hierarchy, and is sampled by HMC. This
# baseline is its closest CLOSED-FORM analogue: a linear-Gaussian random walk in
# LEVELS with a scalar VAB-weighted observation, whose exact posterior is the
# Kalman/RTS smoother. The contrast is deliberate and documented:
#   * shared: random-walk transition + aggregate observation = genuine evidence;
#   * traded away for closed form: positivity (levels may go slightly negative),
#     heavy-tail robustness, and the cross-sector/innovation-scale hierarchy.
# Retained for comparison only; the canonical engine is the reusable one.
# =============================================================================

#' Kalman filter + RTS smoother for a diagonal random walk with scalar observation
#'
#' State \eqn{\varphi_t\in\mathbb{R}^K} follows \eqn{\varphi_t=\varphi_{t-1}+
#' \mathcal{N}(0,Q)}; observation \eqn{y_t=w_t^\top\varphi_t+\mathcal{N}(0,r)}.
#' Returns the smoothed means and marginal variances.
#' @keywords internal
#' @noRd
.kalman_rts <- function(y, W, m0, P0, Q, r) {
  Tn <- length(y); K <- ncol(W)
  a_pred <- vector("list", Tn); P_pred <- vector("list", Tn)
  a_filt <- vector("list", Tn); P_filt <- vector("list", Tn)

  a_pred[[1]] <- m0; P_pred[[1]] <- P0
  for (t in seq_len(Tn)) {
    w <- W[t, ]
    Pw <- P_pred[[t]] %*% w
    F  <- as.numeric(crossprod(w, Pw)) + r
    v  <- y[t] - sum(w * a_pred[[t]])
    Kg <- Pw / F
    a_filt[[t]] <- a_pred[[t]] + as.numeric(Kg) * v
    P_filt[[t]] <- P_pred[[t]] - (Pw %*% t(Pw)) / F
    P_filt[[t]] <- (P_filt[[t]] + t(P_filt[[t]])) / 2          # symmetrize
    if (t < Tn) {
      a_pred[[t + 1]] <- a_filt[[t]]
      P_pred[[t + 1]] <- P_filt[[t]] + Q
    }
  }

  a_sm <- vector("list", Tn); P_sm <- vector("list", Tn)
  a_sm[[Tn]] <- a_filt[[Tn]]; P_sm[[Tn]] <- P_filt[[Tn]]
  for (t in (Tn - 1):1) {
    if (t < 1) break
    J <- P_filt[[t]] %*% solve(P_pred[[t + 1]])
    a_sm[[t]] <- a_filt[[t]] + J %*% (a_sm[[t + 1]] - a_pred[[t + 1]])
    P_sm[[t]] <- P_filt[[t]] + J %*% (P_sm[[t + 1]] - P_pred[[t + 1]]) %*% t(J)
    P_sm[[t]] <- (P_sm[[t]] + t(P_sm[[t]])) / 2
  }

  mean_mat <- do.call(rbind, lapply(a_sm, as.numeric))
  var_mat  <- do.call(rbind, lapply(P_sm, function(P) pmax(diag(P), 0)))
  list(mean = mean_mat, var = var_mat, P_sm = P_sm)
}

#' Conjugate (closed-form) disaggregation baseline
#'
#' Exact linear-Gaussian state-space posterior (Kalman/RTS smoother) for the
#' sectoral price-index levels \eqn{\varphi_{t,k}} given the aggregate index and
#' the VAB weights. Optionally returns joint posterior draws via the
#' Durbin-Koopman simulation smoother (so the draws can also feed
#' \code{bayesianOU::fit_ou_nested_mi}), and a pointwise Gaussian log-likelihood.
#'
#' @param cpi Numeric vector (length \eqn{T}); observed aggregate index (levels).
#' @param W Numeric matrix (\eqn{T \times K}); VAB weights (rows sum to 1).
#' @param years,industries Optional period and sector labels.
#' @param q_frac Random-walk innovation sd as a fraction of \code{sd(cpi)}
#'   (state noise). Default \code{0.10}.
#' @param r_frac Observation sd as a fraction of \code{sd(cpi)}. Default \code{0.05}.
#' @param p0_frac Initial cross-sectional sd as a fraction of \code{cpi[1]}.
#'   Default \code{0.30}.
#' @param n_draws Integer; number of joint posterior draws (simulation smoother).
#'   \code{0} (default) returns only the smoothed mean and bands.
#' @param seed Integer RNG seed (used only when \code{n_draws > 0}).
#'
#' @return An object of class \code{"disagg_conjugate"}: a list with
#'   \code{phi_summary} (median = smoothed mean, \code{q2.5}/\code{q97.5} from the
#'   marginal Gaussian), \code{agg_summary}, \code{loglik} (total Gaussian
#'   log-likelihood of \code{cpi}), \code{phi_draws} (\code{[T, K, n_draws]} or
#'   \code{NULL}), \code{years}, \code{industries}, \code{config}.
#'
#' @seealso \code{\link{disaggregate_statespace}} (canonical engine).
#' @examples
#' sim <- simulate_disagg(T = 25, K = 4, seed = 7)
#' bl  <- disaggregate_conjugate(sim$cpi, sim$W)
#' dim(bl$phi_summary$median)
#' @export
disaggregate_conjugate <- function(cpi, W, years = NULL, industries = NULL,
                                   q_frac = 0.10, r_frac = 0.05, p0_frac = 0.30,
                                   n_draws = 0L, seed = 1234L) {
  cpi <- as.numeric(cpi)
  if (length(cpi) < 2L || any(!is.finite(cpi)) || any(cpi <= 0)) {
    stop("`cpi` must be a finite, strictly positive numeric vector of length >= 2.",
         call. = FALSE)
  }
  W <- row_norm1(as.matrix(W))
  Tn <- length(cpi); K <- ncol(W)
  if (nrow(W) != Tn) stop("`W` rows must match length(cpi).", call. = FALSE)

  industries <- industries %||% colnames(W) %||% paste0("sector_", seq_len(K))
  years <- years %||% seq_len(Tn)

  sd_cpi <- stats::sd(cpi)
  r  <- max(.Machine$double.eps, (r_frac * sd_cpi)^2)
  Q  <- diag((q_frac * sd_cpi)^2, K)
  m0 <- rep(cpi[1], K)
  P0 <- diag((p0_frac * cpi[1])^2, K)

  sm <- .kalman_rts(cpi, W, m0, P0, Q, r)
  med <- sm$mean; sdv <- sqrt(sm$var)
  lo <- med - 1.959964 * sdv; hi <- med + 1.959964 * sdv
  dimnames(med) <- dimnames(lo) <- dimnames(hi) <- list(years, industries)

  agg_mean <- rowSums(W * med)
  agg_var  <- vapply(seq_len(Tn), function(t) {
    as.numeric(crossprod(W[t, ], sm$P_sm[[t]] %*% W[t, ]))
  }, numeric(1))
  agg_summary <- cbind(q2.5 = agg_mean - 1.959964 * sqrt(agg_var),
                       median = agg_mean,
                       q97.5 = agg_mean + 1.959964 * sqrt(agg_var))
  rownames(agg_summary) <- years

  # Gaussian total log-likelihood of the aggregate at the smoothed fit.
  loglik <- sum(stats::dnorm(cpi, agg_mean, sqrt(agg_var + r), log = TRUE))

  phi_draws <- NULL
  n_draws <- as.integer(n_draws)
  if (n_draws > 0L) {
    phi_draws <- .simulation_smoother(cpi, W, m0, P0, Q, r, sm$mean, n_draws, seed)
    dimnames(phi_draws) <- list(years, industries, NULL)
  }

  out <- list(
    phi_summary = list(median = med, q2.5 = lo, q97.5 = hi),
    agg_summary = agg_summary,
    loglik = loglik,
    phi_draws = phi_draws,
    cpi = cpi, W = W, years = years, industries = industries,
    config = list(T = Tn, K = K, q_frac = q_frac, r_frac = r_frac,
                  p0_frac = p0_frac, n_draws = n_draws)
  )
  class(out) <- c("disagg_conjugate", "list")
  out
}

#' Durbin-Koopman simulation smoother for the linear-Gaussian RW state space
#'
#' Returns joint posterior draws \code{[T, K, n_draws]} of \eqn{\varphi} with the
#' correct cross-time/cross-sector covariance (not marginal approximations).
#' @keywords internal
#' @noRd
.simulation_smoother <- function(y, W, m0, P0, Q, r, a_sm_real, n_draws, seed) {
  set.seed(seed)
  Tn <- length(y); K <- ncol(W)
  cP0 <- chol(P0); cQ <- chol(Q)
  out <- array(NA_real_, dim = c(Tn, K, n_draws))
  for (d in seq_len(n_draws)) {
    phi_plus <- matrix(NA_real_, Tn, K)
    phi_plus[1, ] <- m0 + as.numeric(crossprod(cP0, stats::rnorm(K)))
    for (t in 2:Tn) phi_plus[t, ] <- phi_plus[t - 1, ] + as.numeric(crossprod(cQ, stats::rnorm(K)))
    y_plus <- vapply(seq_len(Tn), function(t) {
      sum(W[t, ] * phi_plus[t, ]) + stats::rnorm(1, 0, sqrt(r))
    }, numeric(1))
    sm_plus <- .kalman_rts(y_plus, W, m0, P0, Q, r)
    out[, , d] <- phi_plus - sm_plus$mean + a_sm_real
  }
  out
}

#' @export
print.disagg_conjugate <- function(x, ...) {
  cat("<disagg_conjugate>  closed-form linear-Gaussian baseline (Kalman/RTS)\n")
  cat(sprintf("  periods T = %d, sectors K = %d, joint draws = %d\n",
              x$config$T, x$config$K, x$config$n_draws))
  cat(sprintf("  aggregate Gaussian log-likelihood = %.2f\n", x$loglik))
  invisible(x)
}

# Closed-form conjugate baseline (Kalman/RTS) — pure R, always runs.

test_that("simulate_disagg is deterministic and well-formed", {
  s1 <- simulate_disagg(T = 20, K = 3, seed = 5)
  s2 <- simulate_disagg(T = 20, K = 3, seed = 5)
  expect_identical(s1$cpi, s2$cpi)
  expect_identical(s1$phi_true, s2$phi_true)
  expect_equal(dim(s1$phi_true), c(20L, 3L))
  expect_true(all(s1$cpi > 0))
  expect_true(all(abs(rowSums(s1$W) - 1) < 1e-10))   # weights on the simplex
  expect_equal(s1$agg_true, rowSums(s1$W * s1$phi_true), tolerance = 1e-12)
})

test_that("conjugate disaggregation has correct structure and tracks the aggregate", {
  sim <- simulate_disagg(T = 25, K = 4, seed = 7)
  bl  <- disaggregate_conjugate(sim$cpi, sim$W)
  expect_s3_class(bl, "disagg_conjugate")
  med <- bl$phi_summary$median
  expect_equal(dim(med), c(25L, 4L))
  # credible bands ordered
  expect_true(all(bl$phi_summary$q2.5 <= med))
  expect_true(all(med <= bl$phi_summary$q97.5))
  # the smoothed aggregate reproduces sum_k W phi and tracks the CPI tightly
  expect_equal(bl$agg_summary[, "median"], rowSums(sim$W * med), tolerance = 1e-8)
  expect_gt(cor(bl$agg_summary[, "median"], sim$cpi), 0.95)
  expect_true(is.finite(bl$loglik))
})

test_that("K = 1 with unit weight: smoother essentially observes the level", {
  set.seed(1)
  Tn <- 30
  cpi <- 100 + cumsum(rnorm(Tn, 0, 1))
  W <- matrix(1, Tn, 1)
  bl <- disaggregate_conjugate(cpi, W, r_frac = 0.02)
  # with a direct (unit-weight) observation and small obs noise the smoothed
  # state must hug the data
  expect_gt(cor(bl$phi_summary$median[, 1], cpi), 0.98)
})

test_that("simulation smoother returns reproducible joint draws centred on the smooth", {
  sim <- simulate_disagg(T = 20, K = 3, seed = 9)
  b1 <- disaggregate_conjugate(sim$cpi, sim$W, n_draws = 200, seed = 123)
  b2 <- disaggregate_conjugate(sim$cpi, sim$W, n_draws = 200, seed = 123)
  expect_equal(dim(b1$phi_draws), c(20L, 3L, 200L))
  expect_identical(b1$phi_draws, b2$phi_draws)               # seeded -> reproducible
  # Durbin-Koopman draws are unbiased for the smoothed mean
  draw_mean <- apply(b1$phi_draws, c(1, 2), mean)
  expect_equal(draw_mean, b1$phi_summary$median, tolerance = 0.5)
})

test_that("conjugate draws satisfy the bayesianOU phi_draws array contract", {
  sim <- simulate_disagg(T = 18, K = 2, seed = 3)
  bl <- disaggregate_conjugate(sim$cpi, sim$W, n_draws = 10, seed = 1)
  expect_true(is.array(bl$phi_draws))
  expect_length(dim(bl$phi_draws), 3L)
  expect_equal(dim(bl$phi_draws)[1:2], c(18L, 2L))
})

# =============================================================================
# Canonical evidence-based disaggregation engine (state-space, Model A).
# This is the single, reusable implementation: convergenceDFM imports it instead
# of carrying its own deterministic blend.
# =============================================================================

#' Default prior scales for the state-space disaggregation
#'
#' Weakly-informative scales on the raw index level, derived from the observed
#' aggregate so the model is location/scale aware. Override any field by passing
#' a (partial) named list to \code{disaggregate_statespace(priors = ...)}.
#'
#' @param cpi Numeric vector; the observed aggregate index (level).
#' @return Named list of prior scales (see \code{\link{disaggregate_statespace}}).
#' @export
disagg_default_priors <- function(cpi) {
  if (!is.numeric(cpi) || length(cpi) < 2L || any(!is.finite(cpi)) || any(cpi <= 0)) {
    stop("`cpi` must be a finite, strictly positive numeric vector of length >= 2.",
         call. = FALSE)
  }
  sd_cpi <- stats::sd(cpi)
  list(
    phi1_center     = cpi[1],            # anchor the t = 1 cross-section at the aggregate
    s_omega_struct  = 0.5,               # cross-sectional log-level dispersion (estimable)
    s_delta_mu      = 0.1,               # common annual log-drift sd
    s_delta_sigma   = 0.1,               # drift dispersion across sectors
    log_tau_loc     = log(0.05),         # innovation scale ~ 5% per period (log scale)
    s_log_tau_mu    = 1.0,
    s_log_tau_sigma = 0.5,
    s_sigma_cpi     = max(.Machine$double.eps, 0.1 * sd_cpi)  # observation noise scale
  )
}

#' Evidence-based Bayesian disaggregation (state-space; canonical engine)
#'
#' Disaggregates an observed aggregate index (CPI) into \eqn{K} latent sectoral
#' price indices \eqn{\varphi_{t,k}} with a Bayesian state-space model in which
#' the aggregate enters as a genuine observation density (not a renormalization
#' identity). The model couples a random-walk-with-drift transition in
#' \eqn{\log\varphi} (partial pooling on the drift and the innovation scale), an
#' estimable cross-sectional concentration, and a Student-t (or Gaussian)
#' observation \eqn{cpi_t \mid \varphi \sim \mathrm{Student\text{-}t}(\nu,
#' \sum_k W_{t,k}\varphi_{t,k}, \sigma)}. See \code{vignette("evidence-based-disaggregation")}.
#'
#' The returned posterior draws of \eqn{\varphi} (a \code{[T, K, draws]} array)
#' are exactly the multiple-imputation input consumed by
#' \code{bayesianOU::fit_ou_nested_mi()}, propagating the disaggregation
#' uncertainty into the downstream nested-OU analysis (Rubin's rule).
#'
#' @param cpi Numeric vector (length \eqn{T}); the observed aggregate index in
#'   levels (e.g. CPI re-indexed to a common base). Strictly positive.
#' @param W Numeric matrix (\eqn{T \times K}); the (known) VAB aggregation
#'   weights, rows summing to 1 (small deviations are renormalized). Sector
#'   columns must align with the desired output ordering.
#' @param years Optional integer vector (length \eqn{T}); period labels.
#' @param industries Optional character vector (length \eqn{K}); sector labels.
#'   Defaults to \code{colnames(W)} when present.
#' @param student_obs Logical; if \code{TRUE} (default) the observation is
#'   Student-t (robust to aggregate outliers), otherwise Gaussian.
#' @param priors Optional named list overriding \code{\link{disagg_default_priors}}.
#' @param chains,iter,warmup,thin Sampler controls (HMC/NUTS). Defaults
#'   \code{4 / 2000 / 1000 / 1}.
#' @param cores Integer; parallel chains. Default \code{min(chains, detectCores())}.
#' @param adapt_delta,max_treedepth NUTS tuning. Defaults \code{0.95 / 12}.
#' @param seed Integer RNG seed. Default \code{1234}.
#' @param init Sampler init; a numeric scalar is an init radius (cmdstanr) or is
#'   translated to \code{init_r} (rstan). Default \code{0.5}.
#' @param keep_fit Logical; keep the raw Stan fit object in the result. Default
#'   \code{TRUE} (needed to draw further quantities or run LOO).
#' @param verbose Logical; print progress. Default \code{FALSE}.
#'
#' @return An object of class \code{"disagg_statespace"}: a list with
#'   \describe{
#'     \item{phi_draws}{\code{[T, K, draws]} numeric array of posterior draws of
#'       \eqn{\varphi} (the multiple-imputation input for the nested OU).}
#'     \item{phi_summary}{List of \eqn{T \times K} matrices \code{median},
#'       \code{q2.5}, \code{q97.5} (credible bands per sector and period).}
#'     \item{agg_summary}{\eqn{T \times 3} matrix: posterior median and 95\% band
#'       of the fitted aggregate \eqn{\sum_k W\varphi} (against which \code{cpi}
#'       is the evidence).}
#'     \item{years, industries}{Period and sector labels.}
#'     \item{diagnostics}{\code{rhat_max}, \code{divergences}.}
#'     \item{stan_fit}{The Stan fit (if \code{keep_fit}).}
#'     \item{config}{Sampler/prior configuration and \code{T}, \code{K}.}
#'   }
#'
#' @seealso \code{\link{disaggregate_conjugate}} (closed-form Bayesian baseline),
#'   \code{\link{disaggregate_from_files}}, \code{\link{simulate_disagg}}.
#' @examples
#' \dontrun{
#' set.seed(1)
#' sim <- simulate_disagg(T = 30, K = 4)
#' fit <- disaggregate_statespace(sim$cpi, sim$W, chains = 2, iter = 800)
#' dim(fit$phi_draws)            # T x K x draws
#' }
#' @export
disaggregate_statespace <- function(cpi, W, years = NULL, industries = NULL,
                                    student_obs = TRUE, priors = NULL,
                                    chains = 4L, iter = 2000L, warmup = 1000L,
                                    thin = 1L,
                                    cores = NULL, adapt_delta = 0.95,
                                    max_treedepth = 12L, seed = 1234L,
                                    init = 0.5, keep_fit = TRUE, verbose = FALSE) {
  # ---- validate inputs --------------------------------------------------------
  cpi <- as.numeric(cpi)
  if (length(cpi) < 2L || any(!is.finite(cpi)) || any(cpi <= 0)) {
    stop("`cpi` must be a finite, strictly positive numeric vector of length >= 2.",
         call. = FALSE)
  }
  W <- as.matrix(W)
  if (!is.numeric(W) || any(!is.finite(W)) || any(W < 0)) {
    stop("`W` must be a finite, non-negative numeric matrix.", call. = FALSE)
  }
  Tn <- length(cpi); K <- ncol(W)
  if (nrow(W) != Tn) {
    stop(sprintf("`W` has %d rows but `cpi` has length %d; they must match.",
                 nrow(W), Tn), call. = FALSE)
  }
  W <- row_norm1(W)                                   # enforce the simplex per period

  industries <- industries %||% colnames(W) %||% paste0("sector_", seq_len(K))
  if (length(industries) != K) {
    stop("`industries` must have length K = ncol(W).", call. = FALSE)
  }
  years <- years %||% seq_len(Tn)
  if (length(years) != Tn) stop("`years` must have length T = length(cpi).", call. = FALSE)

  cores <- as.integer(cores %||% min(as.integer(chains), parallel::detectCores()))

  # ---- assemble priors and Stan data -----------------------------------------
  pr <- disagg_default_priors(cpi)
  if (!is.null(priors)) {
    if (!is.list(priors)) stop("`priors` must be a named list.", call. = FALSE)
    unknown <- setdiff(names(priors), names(pr))
    if (length(unknown)) {
      stop("Unknown prior fields: ", paste(unknown, collapse = ", "), call. = FALSE)
    }
    pr[names(priors)] <- priors
  }

  stan_dat <- .build_stan_dat(cpi, W, pr, student_obs)

  # ---- sample -----------------------------------------------------------------
  fit <- .run_stan_disagg(stan_dat, chains = chains, iter = iter, warmup = warmup,
                          thin = thin, cores = cores, adapt_delta = adapt_delta,
                          max_treedepth = max_treedepth, seed = seed, init = init,
                          verbose = verbose)

  # ---- extract ----------------------------------------------------------------
  phi_arr <- .matrix_param_to_array(fit, "phi", Tn, K)
  if (is.null(phi_arr)) stop("Could not extract `phi` draws from the Stan fit.", call. = FALSE)
  dimnames(phi_arr) <- list(years, industries, NULL)

  phi_summary <- .summarize_phi(phi_arr, years, industries)
  agg_summary <- .summarize_agg(fit, Tn, years)

  out <- list(
    phi_draws   = phi_arr,
    phi_summary = phi_summary,
    agg_summary = agg_summary,
    cpi         = cpi,
    W           = W,
    years       = years,
    industries  = industries,
    diagnostics = .stan_diagnostics(fit),
    stan_fit    = if (keep_fit) fit else NULL,
    config      = list(T = Tn, K = K, student_obs = isTRUE(student_obs),
                       priors = pr, chains = chains, iter = iter, warmup = warmup,
                       thin = thin, seed = seed, n_draws = dim(phi_arr)[3])
  )
  class(out) <- c("disagg_statespace", "list")
  out
}

#' Assemble the Stan data list from inputs and resolved priors
#'
#' Single source of truth for the \code{stan_dat} layout, shared by the fit
#' driver and the golden generator (so the frozen fixture uses exactly the data
#' the model is fitted with).
#' @keywords internal
#' @noRd
.build_stan_dat <- function(cpi, W, pr, student_obs) {
  list(
    T = length(cpi), K = ncol(W), cpi = cpi, W = W,
    phi1_center = pr$phi1_center, s_omega_struct = pr$s_omega_struct,
    s_delta_mu = pr$s_delta_mu, s_delta_sigma = pr$s_delta_sigma,
    log_tau_loc = pr$log_tau_loc, s_log_tau_mu = pr$s_log_tau_mu,
    s_log_tau_sigma = pr$s_log_tau_sigma, s_sigma_cpi = pr$s_sigma_cpi,
    student_obs = as.integer(isTRUE(student_obs))
  )
}

#' Summarize phi draws into median / 95% credible bands (T x K matrices)
#' @keywords internal
#' @noRd
.summarize_phi <- function(phi_arr, years, industries) {
  d <- dim(phi_arr); Tn <- d[1]; K <- d[2]
  med <- matrix(NA_real_, Tn, K, dimnames = list(years, industries))
  lo <- med; hi <- med
  for (k in seq_len(K)) {
    q <- t(apply(phi_arr[, k, , drop = TRUE], 1, stats::quantile,
                 probs = c(0.025, 0.5, 0.975), na.rm = TRUE))
    lo[, k] <- q[, 1]; med[, k] <- q[, 2]; hi[, k] <- q[, 3]
  }
  list(median = med, q2.5 = lo, q97.5 = hi)
}

#' Summarize the fitted aggregate sum_k W phi (median + 95% band)
#' @keywords internal
#' @noRd
.summarize_agg <- function(fit, Tn, years) {
  M <- .draws_matrix(fit, "agg")
  if (is.null(M)) return(NULL)
  q <- t(apply(M, 2, stats::quantile, probs = c(0.025, 0.5, 0.975), na.rm = TRUE))
  rownames(q) <- years; colnames(q) <- c("q2.5", "median", "q97.5")
  q
}

# ---- S3 methods -------------------------------------------------------------

#' @export
print.disagg_statespace <- function(x, ...) {
  cat("<disagg_statespace>  evidence-based state-space disaggregation\n")
  cat(sprintf("  periods T = %d, sectors K = %d, posterior draws = %d\n",
              x$config$T, x$config$K, x$config$n_draws))
  cat(sprintf("  observation: %s\n",
              if (x$config$student_obs) "Student-t (robust)" else "Gaussian"))
  cat(sprintf("  diagnostics: rhat_max = %.3f, divergences = %s\n",
              x$diagnostics$rhat_max %||% NA_real_,
              format(x$diagnostics$divergences %||% NA_real_)))
  invisible(x)
}

#' @export
summary.disagg_statespace <- function(object, ...) {
  med <- object$phi_summary$median
  structure(list(
    T = object$config$T, K = object$config$K,
    industries = object$industries,
    phi_median_range = range(med, na.rm = TRUE),
    diagnostics = object$diagnostics,
    config = object$config
  ), class = "summary.disagg_statespace")
}

#' @export
print.summary.disagg_statespace <- function(x, ...) {
  cat("Evidence-based state-space disaggregation (summary)\n")
  cat(sprintf("  T = %d periods, K = %d sectors\n", x$T, x$K))
  cat(sprintf("  phi (median) range: [%.3f, %.3f]\n",
              x$phi_median_range[1], x$phi_median_range[2]))
  cat(sprintf("  rhat_max = %.3f, divergences = %s\n",
              x$diagnostics$rhat_max %||% NA_real_,
              format(x$diagnostics$divergences %||% NA_real_)))
  invisible(x)
}

# Input validation and small utilities — pure R, always runs (no Stan needed:
# the validation happens before any sampling).

test_that("disaggregate_statespace rejects malformed inputs before sampling", {
  W <- matrix(1/3, 5, 3)
  expect_error(disaggregate_statespace(c(1), W), "length >= 2")
  expect_error(disaggregate_statespace(c(-1, 2, 3, 4, 5), W), "strictly positive")
  expect_error(disaggregate_statespace(rep(100, 4), W), "must match")          # T mismatch
  expect_error(disaggregate_statespace(rep(100, 5), W, industries = c("a", "b")),
               "length K")
  expect_error(disaggregate_statespace(rep(100, 5), W, years = 1:4), "length T")
  expect_error(disaggregate_statespace(rep(100, 5), W, priors = list(nope = 1)),
               "Unknown prior")
})

test_that("disagg_default_priors validates the aggregate and scales with it", {
  expect_error(disagg_default_priors(c(1)), "length >= 2")
  expect_error(disagg_default_priors(c(0, 1, 2)), "strictly positive")
  pr <- disagg_default_priors(c(100, 110, 120, 130))
  expect_equal(pr$phi1_center, 100)
  expect_true(pr$s_sigma_cpi > 0)
  expect_named(pr, c("phi1_center", "s_omega_struct", "s_delta_mu", "s_delta_sigma",
                     "log_tau_loc", "s_log_tau_mu", "s_log_tau_sigma", "s_sigma_cpi"))
})

test_that("disaggregate_conjugate rejects malformed inputs", {
  W <- matrix(1/3, 5, 3)
  expect_error(disaggregate_conjugate(c(1), W), "length >= 2")
  expect_error(disaggregate_conjugate(c(-1, 1, 1, 1, 1), W), "strictly positive")
  expect_error(disaggregate_conjugate(rep(100, 4), W), "match length")
})

test_that("row_norm1 normalizes rows and is robust to degenerate rows", {
  rn <- BayesianDisaggregation:::row_norm1
  M <- matrix(c(1, 1, 2, 2), 2, byrow = TRUE)
  expect_equal(rowSums(rn(M)), c(1, 1))
  M0 <- matrix(c(0, 0, 1, 1), 2, byrow = TRUE)          # first row all-zero
  out <- rn(M0)
  expect_true(all(is.finite(out)))
})

test_that("to_num_commas parses localized numerics", {
  f <- BayesianDisaggregation:::to_num_commas
  expect_equal(f("1.234,5"), 1234.5)
  expect_equal(f(c(10, 20)), c(10, 20))
})

# =============================================================================
# Synthetic data generator for the state-space disaggregation model.
# Used by the recovery tests, the goldens and the documentation examples. The
# DGP is the SAME as the Stan model (random-walk-with-drift in log phi, partial
# pooling on drift and innovation scale, VAB-weighted aggregate observation), so
# parameter/path recovery is a well-posed question.
# =============================================================================

#' Simulate from the state-space disaggregation DGP
#'
#' Generates a synthetic aggregate index \code{cpi}, the (known) VAB weights
#' \code{W}, and the latent sectoral price-index paths \code{phi_true} from the
#' same data-generating process as \code{\link{disaggregate_statespace}}. The
#' innovation scale is kept modest so the log random walk stays in a numerically
#' stable region over the simulated horizon (the same care taken in the sibling
#' OU simulator).
#'
#' @param T Integer; number of periods.
#' @param K Integer; number of sectors.
#' @param phi1_center Numeric; central level of the initial cross-section.
#' @param omega_struct Numeric; cross-sectional log-level dispersion at t = 1.
#' @param delta_mu,delta_sigma Common drift and its cross-sector dispersion.
#' @param tau_mu,tau_sigma Geometric mean innovation scale and log-dispersion
#'   (so \eqn{\tau_k = \tau_{mu}\exp(\tau_{sigma} z_k)}).
#' @param sigma_cpi Observation noise scale on the aggregate.
#' @param nu Student-t degrees of freedom of the observation (\code{Inf} = Gaussian).
#' @param seed Integer RNG seed.
#'
#' @return A list with \code{cpi} (length \eqn{T}), \code{W} (\eqn{T \times K},
#'   rows sum to 1), \code{phi_true} (\eqn{T \times K}), \code{agg_true}
#'   (length \eqn{T}), and \code{params} (the true scalar/vector parameters).
#' @examples
#' sim <- simulate_disagg(T = 20, K = 3, seed = 42)
#' str(sim$params)
#' @export
simulate_disagg <- function(T = 40L, K = 5L, phi1_center = 100,
                            omega_struct = 0.3,
                            delta_mu = 0.02, delta_sigma = 0.01,
                            tau_mu = 0.04, tau_sigma = 0.3,
                            sigma_cpi = 1.0, nu = Inf, seed = 1234L) {
  T <- as.integer(T); K <- as.integer(K)
  if (T < 2L || K < 1L) stop("Need T >= 2 and K >= 1.", call. = FALSE)
  set.seed(seed)

  delta <- delta_mu + delta_sigma * stats::rnorm(K)
  tau   <- tau_mu * exp(tau_sigma * stats::rnorm(K))

  log_phi <- matrix(NA_real_, T, K)
  log_phi[1, ] <- log(phi1_center) + omega_struct * stats::rnorm(K)
  for (t in 2:T) {
    log_phi[t, ] <- log_phi[t - 1, ] + delta + tau * stats::rnorm(K)
  }
  phi_true <- exp(log_phi)

  # VAB weights: a smooth, persistent simplex per period (Dirichlet-ish via
  # a slowly drifting log-share), so they are realistic but known.
  base_share <- stats::runif(K, 0.5, 1.5)
  log_share <- matrix(log(base_share), T, K, byrow = TRUE) +
    apply(matrix(stats::rnorm(T * K, sd = 0.02), T, K), 2, cumsum)
  W <- exp(log_share)
  W <- W / rowSums(W)

  agg_true <- rowSums(W * phi_true)
  noise <- if (is.finite(nu)) sigma_cpi * stats::rt(T, df = nu) else stats::rnorm(T, sd = sigma_cpi)
  cpi <- agg_true + noise
  cpi <- pmax(cpi, .Machine$double.eps)             # keep strictly positive

  list(
    cpi = as.numeric(cpi), W = W, phi_true = phi_true, agg_true = agg_true,
    params = list(delta = delta, tau = tau, delta_mu = delta_mu,
                  delta_sigma = delta_sigma, tau_mu = tau_mu, tau_sigma = tau_sigma,
                  omega_struct = omega_struct, sigma_cpi = sigma_cpi, nu = nu,
                  phi1_center = phi1_center, T = T, K = K)
  )
}

# =============================================================================
# Convenience reader: build the state-space disaggregation directly from the
# CPI and VAB-weights Excel files, reusing the existing robust readers and
# aligning on the common years.
# =============================================================================

#' Align CPI and VAB weights on their common years
#'
#' Reads the CPI and the weights matrix, intersects their years, and returns the
#' aligned aggregate vector and weight matrix ready for the disaggregation
#' engines. Both engines (state-space and conjugate) consume this output.
#'
#' @param path_cpi Path to the CPI Excel file (see \code{\link{read_cpi}}).
#' @param path_weights Path to the VAB-weights Excel file
#'   (see \code{\link{read_weights_matrix}}).
#' @return A list with \code{cpi} (numeric, length \eqn{T}), \code{W}
#'   (\eqn{T \times K}, rows sum to 1), \code{years}, \code{industries}.
#' @seealso \code{\link{disaggregate_from_files}}
#' @export
align_disagg_inputs <- function(path_cpi, path_weights) {
  cpi_df <- read_cpi(path_cpi)
  Wraw   <- read_weights_matrix(path_weights)
  Pfull  <- Wraw$P

  years_common <- sort(unique(intersect(as.integer(rownames(Pfull)), cpi_df$Year)))
  if (length(years_common) < 2L) {
    stop("Insufficient common years between CPI and weights data.", call. = FALSE)
  }
  W   <- Pfull[as.character(years_common), , drop = FALSE]
  cpi <- cpi_df$CPI[match(years_common, cpi_df$Year)]

  list(cpi = as.numeric(cpi), W = W, years = years_common,
       industries = colnames(W))
}

#' Evidence-based disaggregation directly from Excel files
#'
#' Thin convenience wrapper: reads and aligns the CPI and VAB-weight files
#' (\code{\link{align_disagg_inputs}}) and runs the canonical state-space engine
#' (\code{\link{disaggregate_statespace}}).
#'
#' @param path_cpi Path to the CPI Excel file (index levels, re-indexed to the
#'   same base as the production prices; see the package vignette and the data
#'   note on \code{convert_to_index}).
#' @param path_weights Path to the VAB-weights Excel file.
#' @param ... Passed to \code{\link{disaggregate_statespace}} (sampler controls,
#'   \code{priors}, \code{student_obs}, ...).
#' @return A \code{"disagg_statespace"} object.
#' @examples
#' \dontrun{
#' cpi_file <- system.file("extdata", "CPI.xlsx", package = "BayesianDisaggregation")
#' w_file   <- system.file("extdata", "WEIGHTS.xlsx", package = "BayesianDisaggregation")
#' fit <- disaggregate_from_files(cpi_file, w_file, chains = 2, iter = 800)
#' }
#' @seealso \code{\link{disaggregate_statespace}}, \code{\link{align_disagg_inputs}}
#' @export
disaggregate_from_files <- function(path_cpi, path_weights, ...) {
  al <- align_disagg_inputs(path_cpi, path_weights)
  disaggregate_statespace(cpi = al$cpi, W = al$W, years = al$years,
                          industries = al$industries, ...)
}

# ----- Internal package state -------------------------------------------------
# Private environment for package-level state (no writes to package namespace)
.bd_env <- new.env(parent = emptyenv())
.bd_env$log_level <- "INFO"

# Log levels (ordering)
.level_order <- c("TRACE" = 1, "DEBUG" = 2, "INFO" = 3, "WARN" = 4, "ERROR" = 5)

#' Enable logging at a specific level
#'
#' Sets the package-wide logging verbosity.
#'
#' @param level Character scalar. One of "TRACE", "DEBUG", "INFO", "WARN", "ERROR".
#' @return (Invisibly) the level set.
#' @export
log_enable <- function(level = "INFO") {
  if (!is.character(level) || length(level) != 1L || !level %in% names(.level_order)) {
    stop('Invalid log level. Choose one of: "TRACE","DEBUG","INFO","WARN","ERROR".')
  }
  .bd_env$log_level <- level
  invisible(level)
}

#' Log message with timestamp
#'
#' Internal helper that prints a timestamped message when the current log
#' level is at least \code{level}.
#'
#' @param level Character level: "TRACE","DEBUG","INFO","WARN","ERROR".
#' @param ...   Message components (will be concatenated with spaces).
#' @keywords internal
log_msg <- function(level = "INFO", ...) {
  cur <- if (!is.null(.bd_env$log_level)) .bd_env$log_level else "INFO"
  if (.level_order[[level]] >= .level_order[[cur]]) {
    ts <- format(Sys.time(), "%Y-%m-%d %H:%M:%S")
    message(sprintf("[%s][%s] %s", ts, level, paste(..., collapse = " ")))
  }
}

# ----- Mathematical utilities -------------------------------------------------

#' Row-wise L1 normalization (safe)
#' Ensures each row sums to 1; protects against NaN/Inf and near-zero sums.
#' @keywords internal
#' @noRd
row_norm1 <- function(X, eps = .Machine$double.eps) {
  rs <- rowSums(X)
  rs[!is.finite(rs) | rs < eps] <- 1
  X / rs
}

#' Parse localized numerics (thousands as ".", decimal as ",")
#' @keywords internal
#' @noRd
to_num_commas <- function(x) {
  if (is.numeric(x)) return(as.numeric(x))
  x <- stringr::str_replace_all(as.character(x), "\\.", "")
  x <- stringr::str_replace_all(x, ",", ".")
  suppressWarnings(as.numeric(x))
}

# NSE column names used by the tidyverse data readers (read_cpi / read_weights_matrix).
utils::globalVariables(c(".", "Year", "CPI", "Industry", "Weight_raw", "Weight"))

#' Read CPI data from an Excel file
#'
#' Loads and normalizes a CPI time series from an Excel worksheet. The function
#' detects the date/year column and the CPI/value column by pattern-matching on
#' lower-cased header names, parses localized numerics (via \code{to_num_commas()}),
#' collapses duplicate years by averaging, and returns a clean, sorted data frame.
#'
#' @param path_cpi Character path to the CPI Excel file.
#'
#' @details
#' \strong{Column detection.} Headers are lower-cased and matched with:
#' \itemize{
#'   \item Date/year: patterns \code{"date|fecha|year|anio|ano"}.
#'   \item CPI/value: patterns \code{"cpi|indice|price"}.
#' }
#' If either column cannot be identified, the function errors.
#'
#' \strong{Cleaning.}
#' \itemize{
#'   \item Year is extracted as the first 4 digits of the date-like column.
#'   \item CPI is parsed with \code{to_num_commas()} (handles commas/thousands).
#'   \item \code{NA} rows are dropped; duplicates in \code{Year} are averaged.
#'   \item Output is sorted by \code{Year} ascending.
#' }
#'
#' @return A \code{data.frame} with two columns:
#' \itemize{
#'   \item \code{Year} (\code{integer})
#'   \item \code{CPI}  (\code{numeric})
#' }
#'
#' @examples
#' cpi_file <- system.file("extdata", "CPI.xlsx", package = "BayesianDisaggregation")
#' if (nzchar(cpi_file)) {
#'   df <- read_cpi(cpi_file)
#'   head(df)
#' }
#'
#' @seealso \code{\link{read_weights_matrix}}, \code{\link{align_disagg_inputs}}
#' @export
read_cpi <- function(path_cpi) {
  log_msg("INFO", "Reading CPI:", path_cpi)
  df <- readxl::read_excel(path_cpi)
  cn <- tolower(names(df))
  # `which` (not `which.max`): on NO match str_detect is all-FALSE and which.max
  # silently returns index 1 (a false positive on the first column); `which`
  # returns integer(0) so the length check below errors honestly. On AMBIGUOUS
  # matches which.max takes only the first; `which` returns all so the length
  # check flags the ambiguity (deuda D-7.2, saldada en Sesion 10).
  col_date <- which(stringr::str_detect(cn, "date|fecha|year|anio|ano"))
  col_cpi  <- which(stringr::str_detect(cn, "cpi|indice|price"))

  if (length(col_date) != 1 || length(col_cpi) != 1) {
    stop("Could not identify date and CPI columns in the CPI file")
  }

  df <- df %>%
    dplyr::transmute(
      Year = as.integer(substr(as.character(.[[col_date]]), 1, 4)),
      CPI  = to_num_commas(.[[col_cpi]])
    ) %>%
    dplyr::filter(!is.na(Year), !is.na(CPI)) %>%
    dplyr::arrange(Year)

  if (anyDuplicated(df$Year)) {
    log_msg("WARN", "CPI with duplicate years; aggregating by average")
    df <- df %>%
      dplyr::group_by(Year) %>%
      dplyr::summarise(CPI = mean(CPI), .groups = "drop")
  }

  df
}

#' Read a weights matrix from an Excel file
#'
#' Loads a sector-by-year weight table, normalizes weights to the simplex per year,
#' and returns a list with the \eqn{T \times K} prior matrix \code{P}, the sector
#' names, and the year vector. The first column is assumed to contain sector names
#' (renamed to \code{Industry}); all other columns are treated as years.
#'
#' @param path_weights Character path to the weights Excel file.
#'
#' @details
#' \strong{Expected layout.} One sheet with:
#' \itemize{
#'   \item First column: sector names (any header; renamed to \code{Industry}).
#'   \item Remaining columns: years; the function extracts a 4-digit year from each
#'         header using \code{stringr::str_extract(Year, "\\\\d{4}")}.
#' }
#' Values are parsed with \code{to_num_commas()}, missing rows are dropped, and
#' weights are normalized within each year to sum to 1. Any absent (sector, year)
#' entry becomes 0 when pivoting wide. Finally, rows are re-normalized with
#' \code{row_norm1()} for numerical safety.
#'
#' \strong{Safeguards.}
#' \itemize{
#'   \item Rows with all-missing/zero after parsing are dropped by the filters.
#'   \item If no valid year columns are found, the function errors.
#' }
#'
#' @return A \code{list} with:
#' \describe{
#'   \item{\code{P}}{\eqn{T \times K} numeric matrix of prior weights (rows sum to 1).}
#'   \item{\code{industries}}{Character vector of sector names (length \eqn{K}).}
#'   \item{\code{years}}{Integer vector of years (length \eqn{T}).}
#' }
#'
#' @examples
#' w_file <- system.file("extdata", "WEIGHTS.xlsx", package = "BayesianDisaggregation")
#' if (nzchar(w_file)) {
#'   w <- read_weights_matrix(w_file)
#'   stopifnot(is.matrix(w$P), all(abs(rowSums(w$P) - 1) < 1e-8))
#'   str(w)
#' }
#'
#' @seealso \code{\link{read_cpi}}, \code{\link{align_disagg_inputs}}
#' @export
read_weights_matrix <- function(path_weights) {
  log_msg("INFO", "Reading weights matrix:", path_weights)
  raw <- readxl::read_excel(path_weights)
  names(raw)[1] <- "Industry"

  long <- raw %>%
    tidyr::pivot_longer(-Industry, names_to = "Year", values_to = "Weight_raw") %>%
    dplyr::mutate(
      Year   = as.integer(stringr::str_extract(Year, "\\d{4}")),
      Weight = to_num_commas(Weight_raw)
    ) %>%
    dplyr::filter(!is.na(Year), !is.na(Weight))

  long <- long %>%
    dplyr::group_by(Year) %>%
    dplyr::mutate(Weight = Weight / sum(Weight, na.rm = TRUE)) %>%
    dplyr::ungroup()

  wide <- long %>%
    dplyr::select(Industry, Year, Weight) %>%
    tidyr::pivot_wider(names_from = Industry, values_from = Weight, values_fill = 0) %>%
    dplyr::arrange(Year)

  P <- as.matrix(wide[, -1, drop = FALSE])
  rownames(P) <- wide$Year
  P <- row_norm1(P)

  list(P = P, industries = colnames(P), years = as.integer(rownames(P)))
}

# Data readers and alignment against the bundled extdata — always runs.

cpi_file <- function() system.file("extdata", "CPI.xlsx", package = "BayesianDisaggregation")
w_file   <- function() system.file("extdata", "WEIGHTS.xlsx", package = "BayesianDisaggregation")

test_that("read_cpi returns a clean, sorted Year/CPI frame", {
  skip_if(cpi_file() == "")
  df <- read_cpi(cpi_file())
  expect_true(all(c("Year", "CPI") %in% names(df)))
  expect_false(is.unsorted(df$Year))
  expect_true(all(is.finite(df$CPI)))
  expect_true(all(c(1982, 1983, 1984) %in% df$Year))    # base period present
})

test_that("read_weights_matrix returns a simplex-per-year prior matrix", {
  skip_if(w_file() == "")
  w <- read_weights_matrix(w_file())
  expect_true(is.matrix(w$P))
  expect_true(all(abs(rowSums(w$P) - 1) < 1e-8))
  expect_equal(length(w$industries), ncol(w$P))
  expect_equal(length(w$years), nrow(w$P))
})

test_that("align_disagg_inputs intersects on common years", {
  skip_if(cpi_file() == "" || w_file() == "")
  al <- align_disagg_inputs(cpi_file(), w_file())
  expect_equal(length(al$cpi), nrow(al$W))
  expect_true(all(abs(rowSums(al$W) - 1) < 1e-8))
  expect_true(length(al$years) >= 2)
  expect_equal(al$industries, colnames(al$W))
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## ----conjugate, eval = requireNamespace("BayesianDisaggregation", quietly = TRUE)----
library(BayesianDisaggregation)

sim <- simulate_disagg(T = 30, K = 4, seed = 1)   # synthetic CPI + VAB weights
bl  <- disaggregate_conjugate(sim$cpi, sim$W, n_draws = 100, seed = 1)
bl

## the smoothed aggregate tracks the CPI tightly (aggregate is well identified)
round(cor(bl$agg_summary[, "median"], sim$cpi), 4)

## joint posterior draws: the [T, K, draws] contract consumed by the nested OU
dim(bl$phi_draws)

## ----statespace, eval = FALSE-------------------------------------------------
# fit <- disaggregate_statespace(sim$cpi, sim$W, chains = 4, iter = 2000, warmup = 1000)
# fit$diagnostics                 # rhat_max, divergences
# dim(fit$phi_draws)              # T x K x draws
# str(fit$phi_summary)            # median, q2.5, q97.5 (T x K each)
# 
# ## couple to the nested OU (uncertainty propagated by Rubin's rule):
# ## bayesianOU::fit_ou_nested_mi(phi_draws = fit$phi_draws, X = Phi_index, ...)

## ----fromfiles, eval = FALSE--------------------------------------------------
# cpi_file <- system.file("extdata", "CPI.xlsx", package = "BayesianDisaggregation")
# w_file   <- system.file("extdata", "WEIGHTS.xlsx", package = "BayesianDisaggregation")
# fit <- disaggregate_from_files(cpi_file, w_file, chains = 2, iter = 1000)

---
title: "Sensitivity Analysis Framework for Bayesian Economic Disaggregation"
author: "José Mauricio Gómez Julián"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Sensitivity Analysis Framework}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

 > **How to read this manual.**  
> Sections 1–3 develop the **theory** (with equations); Sections 4–6 give **diagnostics** and **metrics**; Sections 7–8 provide **reproducible code**: a fast synthetic demo (evaluates on knit) and a full **real-data pipeline** (disabled by default for speed, enable by setting `eval=TRUE`). All code is consistent with the functions exported by the `BayesianDisaggregation` package.

```{r setup, include=TRUE}
# Global chunk defaults
knitr::opts_chunk$set(
  echo = TRUE, message = FALSE, warning = FALSE,
  fig.width = 9, fig.height = 6
)

# Libraries
suppressPackageStartupMessages({
  library(BayesianDisaggregation)
  library(dplyr)
  library(tidyr)
  library(ggplot2)
  library(readr)
  library(openxlsx)
})

# Logging verbosity from the package
log_enable("INFO")
set.seed(2024)
```

# 1. Problem Setup

We observe an **aggregate index** (e.g., CPI) by period $t=1,\dots,T$, and we want a **sectoral disaggregation** into $K$ components whose period-wise shares lie on the **unit simplex**:

$$
W_t = (w_{t1},\dots,w_{tK}),\qquad w_{tk}\ge 0,\quad \sum_{k=1}^K w_{tk}=1.
$$

We start with a **prior weight matrix** $P\in\mathbb{R}^{T\times K}$ (rows on the simplex), and construct a **likelihood of sectors** $L\in\Delta^{K-1}$ (a non-negative vector summing to one). A **temporal profile** then spreads $L$ to $LT\in\mathbb{R}^{T\times K}$. Finally, a **deterministic update rule** combines $P$ and $LT$ to obtain the posterior $W$.

# 2. Constructing the Sectoral Likelihood $L$

## 2.1 PCA/SVD of the centered prior matrix

Let $P$ be validated (finite, non-negative, rows $\approx 1$; small deviations renormalized). We **center** columns over time:

$$
X = P - \mathbf{1}\,\bar p^\top,\quad \bar p = \frac{1}{T}\sum_{t=1}^T P_{t\cdot}.
$$

Compute the SVD $X = U\Sigma V^\top$. Let $v$ denote the **first right singular vector** (PC1 loadings). We map to non-negative salience via absolute values and normalize:

$$
\ell_k = |v_k|,\qquad L_k = \frac{\ell_k}{\sum_j \ell_j}.
$$

If PC1 is **degenerate** (near-zero variance or identical columns), we fall back to **column means** of $P$ (renormalized). This is implemented in:

```{r L_from_P, include=TRUE}
# Example call (internals are in the package):
# L <- compute_L_from_P(P) 
```

**Diagnostics attached to `L`:** attributes `"pc1_loadings"`, `"explained_var"`, and `"fallback"`.

## 2.2 Temporal spreading of $L$

We create a time-varying matrix $LT$ by applying a non-negative weight profile $w_t$ and row-renormalizing:

$$
LT_{t,k} \propto w_t L_k,\qquad \sum_k LT_{t,k}=1.
$$

Built-in **patterns**:

* `constant`: $w_t=1$
* `recent`: linearly increasing in $t$ (more weight to recent periods)
* `linear`: affine ramp between endpoints
* `bell`: symmetric Gaussian-like bump around $T/2$

```{r spread-L, include=TRUE}
# Example call:
# LT <- spread_likelihood(L, T_periods = nrow(P), pattern = "recent")
```

# 3. Posterior Updating Rules (Deterministic, MCMC-free)

Given $P$ and $LT$ (both row-wise on the simplex), we define four deterministic updates:

* **Weighted average** (mixing parameter $\lambda\in[0,1]$):

$$
W = \mathsf{norm}_1\{\lambda P + (1-\lambda)LT\}.
$$

* **Multiplicative** (elementwise product with re-normalization):

$$
W = \mathsf{norm}_1\{P\odot LT\}.
$$

* **Dirichlet mean** (analytical conjugacy, $\gamma>0$, smaller $\gamma\Rightarrow$ sharper):

$$
\alpha_{\text{post}} = \frac{P}{\gamma} + \frac{LT}{\gamma},\qquad
W = \frac{\alpha_{\text{post}}}{\mathbf{1}^\top\alpha_{\text{post}}}.
$$

* **Adaptive** (sector-wise mixing by prior volatility):

$$
\phi_k=\min\!\Big(\frac{\sigma_k}{\bar\sigma},\,0.8\Big),\quad
W_{t\cdot}=\mathsf{norm}_1\{(1-\phi)\odot P_{t\cdot} + \phi\odot LT_{t\cdot}\}.
$$

All are exposed in the package:

```{r posteriors, include=TRUE}
# posterior_weighted(P, LT, lambda = 0.7)
# posterior_multiplicative(P, LT)
# posterior_dirichlet(P, LT, gamma = 0.1)
# posterior_adaptive(P, LT)
```

# 4. Coherence, Stability, and Interpretability

## 4.1 Coherence with respect to $L$

Define prior/posterior **temporal means**:

$$
\bar p = \frac{1}{T}\sum_t P_{t\cdot},\qquad \bar w = \frac{1}{T}\sum_t W_{t\cdot}.
$$

Let $\rho(\cdot,\cdot)$ be a **robust correlation** (max of |Pearson| and |Spearman|). The **coherence** scales the **increment** $\Delta\rho=\max(0,\rho(\bar w,L)-\rho(\bar p,L))$:

$$
\text{coherence}=\min\{1,\ \text{const} + \text{mult}\cdot\Delta\rho\}.
$$

## 4.2 Numerical and temporal stability

* **Numerical stability (exponential penalty)** on row-sum deviation and negatives:

$$
S_{\text{num}}=\exp\{-a\cdot\overline{|\sum_k W_{tk}-1|} - b\cdot \#(W<0)\}.
$$

* **Temporal stability** via average $|\Delta|$ (lower variation $\Rightarrow$ higher score):

$$
S_{\text{tmp}} = \frac{1}{1+\kappa\cdot \overline{|\Delta W|}},\quad
\overline{|\Delta W|}=\frac{1}{K}\sum_k \frac{1}{T-1}\sum_{t}|W_{t+1,k}-W_{t,k}|.
$$

* **Composite stability** (default weights 60% numeric, 40% temporal):

$$
S_{\text{comp}} = 0.6\,S_{\text{num}} + 0.4\,S_{\text{tmp}}.
$$

The package functions:

```{r metrics-fns, include=TRUE}
# coherence_score(P, W, L, mult = 3.0, const = 0.5)
# numerical_stability_exp(W, a = 1000, b = 10)
# temporal_stability(W, kappa = 50)
# stability_composite(W, a = 1000, b = 10, kappa = 50)
```

## 4.3 Interpretability

Two principles:

1. **Preservation** of the sectoral structure (correlation between $\bar p$ and $\bar w$);
2. **Plausibility** of average sector changes (penalize extreme relative shifts).

Implementation:

$$
\text{pres}=\max\{0,\rho(\bar p,\bar w)\},\qquad
r_k=\frac{|\,\bar w_k-\bar p_k\,|}{\bar p_k+\epsilon},\quad
\text{plaus}= \frac{1}{1+2\cdot Q_{0.9}(r_k)}.
$$

Then $\text{interp}=0.6\,\text{pres}+0.4\,\text{plaus}$.

```{r interp-fn, include=TRUE}
# interpretability_score(P, W, use_q90 = TRUE)
```

# 5. End-to-End API (`bayesian_disaggregate`)

The convenience pipeline:

1. `read_cpi()` and `read_weights_matrix()` (Excel)
2. `compute_L_from_P(P)` and `spread_likelihood(L, T, pattern)`
3. posterior rule (`weighted` / `multiplicative` / `dirichlet` / `adaptive`)
4. metrics: coherence, stability (composite), interpretability, efficiency (heuristic), composite score
5. export helpers: `save_results()` and a one-file workbook for “best” config

```{r api, include=TRUE}
# Example signature (see Section 8 for real data):
# bayesian_disaggregate(path_cpi, path_weights,
#   method = c("weighted","multiplicative","dirichlet","adaptive"),
#   lambda = 0.7, gamma = 0.1,
#   coh_mult = 3.0, coh_const = 0.5,
#   stab_a = 1000, stab_b = 10, stab_kappa = 50,
#   likelihood_pattern = "recent")
```

# 6. Interpreting Key Visualizations

* **Heatmap of posterior $W$**: each **cell** is a sector share in a year; **rows** are years, **columns** are sectors.
  *Read it as*: darker tiles = larger sector share; **horizontal smoothness** indicates temporal stability; **vertical patterns** (bands) show persistent sectoral importance.

* **Top-sectors lines**: for the most relevant sectors by average share, **lines** track the sector’s share over time. *Read it as*: consistent levels = stability; trend changes coincide with macro structure shifts.

* **Sectoral CPI sheet**: $\hat Y_{t,k} = \text{CPI}_t \times W_{t,k}$.
  *Read it as*: dollarized (or index-scaled) decomposition of the aggregate.

# 7. Reproducible Synthetic Demo (evaluates on knit)

This chunk synthesizes a small example you can knit safely.

```{r demo, include=TRUE}
# Synthetic prior matrix (rows on simplex)
T <- 10; K <- 6
set.seed(123)
P <- matrix(rexp(T*K), nrow = T)
P <- P / rowSums(P)

# Likelihood vector from P (PCA/SVD; robust with fallback)
L  <- compute_L_from_P(P)

# Spread over time with "recent" pattern
LT <- spread_likelihood(L, T_periods = T, pattern = "recent")

# Try a couple of posteriors
W_weighted <- posterior_weighted(P, LT, lambda = 0.7)
W_adaptive <- posterior_adaptive(P, LT)

# Metrics for adaptive
coh  <- coherence_score(P, W_adaptive, L)
stab <- stability_composite(W_adaptive, a = 1000, b = 10, kappa = 50)
intr <- interpretability_score(P, W_adaptive)
eff  <- 0.65
comp <- 0.30*coh + 0.25*stab + 0.25*intr + 0.20*eff

data.frame(coherence = coh, stability = stab, interpretability = intr,
           efficiency = eff, composite = comp) %>% round(4)
```

# 8. Full Real-Data Pipeline (disable/enable evaluation)

> **Switch to `eval=TRUE` after setting your paths**. By default we keep this chunk off to render quickly on any machine.

```{r real-pipeline, eval=FALSE}
# === Create synthetic data for CRAN-compliant demo ===
demo_dir <- tempdir()

# Create synthetic CPI data (mimicking your structure)
set.seed(123)
cpi_demo <- data.frame(
  Year = 2000:2010,
  CPI = cumsum(c(100, rnorm(10, 0.5, 2)))  # Random walk starting at 100
)
cpi_file <- file.path(demo_dir, "synthetic_cpi.xlsx")
openxlsx::write.xlsx(cpi_demo, cpi_file)

# Create synthetic weights matrix (mimicking VAB weights structure)
set.seed(456)
years <- 2000:2010
sectors <- c("Agriculture", "Manufacturing", "Services", "Construction", "Mining")

weights_demo <- data.frame(Year = years)
for(sector in sectors) {
  weights_demo[[sector]] <- runif(length(years), 0.05, 0.35)
}
# Normalize rows to sum to 1 (simplex constraint)
weights_demo[, -1] <- weights_demo[, -1] / rowSums(weights_demo[, -1])
weights_file <- file.path(demo_dir, "synthetic_weights.xlsx")
openxlsx::write.xlsx(weights_demo, weights_file)

# Use synthetic data paths
path_cpi <- cpi_file
path_w <- weights_file
out_dir <- demo_dir

cat("Using synthetic data for CRAN demo:\n")
cat("CPI file:", path_cpi, "\n")
cat("Weights file:", path_w, "\n")
cat("Output directory:", out_dir, "\n")

# --- Base run (robust defaults) ---
base_res <- bayesian_disaggregate(
  path_cpi           = path_cpi,
  path_weights       = path_w,
  method             = "adaptive",
  lambda             = 0.7,   # recorded in metrics; not used by "adaptive"
  gamma              = 0.1,
  coh_mult           = 3.0,
  coh_const          = 0.5,
  stab_a             = 1000,
  stab_b             = 10,
  stab_kappa         = 60,
  likelihood_pattern = "recent"
)
xlsx_base <- save_results(base_res, out_dir = file.path(out_dir, "base"))
print(base_res$metrics)

# --- Minimal grid search for demo (reduced size) ---
n_cores <- 1  # Use single core for CRAN compliance
grid_df <- expand.grid(
  method             = c("weighted", "adaptive"),  # Reduced methods
  lambda             = c(0.5, 0.7),               # Reduced options
  gamma              = 0.1,                       # Single option
  coh_mult           = 3.0,                       # Single option  
  coh_const          = 0.5,                       # Single option
  stab_a             = 1000,
  stab_b             = 10,
  stab_kappa         = 60,                        # Single option
  likelihood_pattern = "recent",                  # Single option
  KEEP.OUT.ATTRS     = FALSE,
  stringsAsFactors   = FALSE
)

grid_res <- run_grid_search(
  path_cpi     = path_cpi,
  path_weights = path_w,
  grid_df      = grid_df,
  n_cores      = n_cores
)
write.csv(grid_res, file.path(out_dir, "grid_results.csv"), row.names = FALSE)

best_row <- grid_res %>% arrange(desc(composite)) %>% slice(1)
print("Best configuration from grid search:")
print(best_row)

# --- Re-run the best configuration for clean export ---
best_res <- bayesian_disaggregate(
  path_cpi           = path_cpi,
  path_weights       = path_w,
  method             = best_row$method,
  lambda             = if (!is.na(best_row$lambda)) best_row$lambda else 0.7,
  gamma              = if (!is.na(best_row$gamma))  best_row$gamma  else 0.1,
  coh_mult           = best_row$coh_mult,
  coh_const          = best_row$coh_const,
  stab_a             = best_row$stab_a,
  stab_b             = best_row$stab_b,
  stab_kappa         = best_row$stab_kappa,
  likelihood_pattern = best_row$likelihood_pattern
)
xlsx_best <- save_results(best_res, out_dir = file.path(out_dir, "best"))

# --- One Excel with everything (including hyperparameters) ---
sector_summary <- tibble(
  Sector          = colnames(best_res$posterior)[-1],
  prior_mean      = colMeans(as.matrix(best_res$prior[, -1])),
  posterior_mean  = colMeans(as.matrix(best_res$posterior[, -1]))
)

wb <- createWorkbook()
addWorksheet(wb, "Hyperparameters"); writeData(wb, "Hyperparameters", best_row)
addWorksheet(wb, "Metrics");         writeData(wb, "Metrics", best_res$metrics)
addWorksheet(wb, "Prior_P");         writeData(wb, "Prior_P", best_res$prior)
addWorksheet(wb, "Posterior_W");     writeData(wb, "Posterior_W", best_res$posterior)
addWorksheet(wb, "Likelihood_t");    writeData(wb, "Likelihood_t", best_res$likelihood_t)
addWorksheet(wb, "Likelihood_L");    writeData(wb, "Likelihood_L", best_res$likelihood)
addWorksheet(wb, "Sector_Summary");  writeData(wb, "Sector_Summary", sector_summary)

for (sh in c("Hyperparameters","Metrics","Prior_P","Posterior_W",
             "Likelihood_t","Likelihood_L","Sector_Summary")) {
  freezePane(wb, sh, firstRow = TRUE)
  addFilter(wb, sh, rows = 1, cols = 1:ncol(readWorkbook(wb, sh)))
  setColWidths(wb, sh, cols = 1:200, widths = "auto")
}

# --- Add sectoral CPI (aggregate times posterior weights) ---
W_post <- best_res$posterior           # Year + sectors
cpi_df <- read_cpi(path_cpi)           # Year, CPI
sector_cpi <- dplyr::left_join(W_post, cpi_df, by = "Year") %>%
  dplyr::mutate(dplyr::across(-c(Year, CPI), ~ .x * CPI))

# Quality check: sector sums vs CPI
check_sum <- sector_cpi %>%
  dplyr::mutate(row_sum = rowSums(dplyr::across(-c(Year, CPI))),
                diff    = CPI - row_sum)
cat("Quality check (first 5 rows):\n")
print(head(check_sum, 5))

addWorksheet(wb, "Sector_CPI")
writeData(wb, "Sector_CPI", sector_cpi)
freezePane(wb, "Sector_CPI", firstRow = TRUE)
addFilter(wb, "Sector_CPI", rows = 1, cols = 1:ncol(sector_cpi))
setColWidths(wb, "Sector_CPI", cols = 1:200, widths = "auto")

excel_onefile <- file.path(out_dir, "best", "Best_Full_Output_withSectorCPI.xlsx")
saveWorkbook(wb, excel_onefile, overwrite = TRUE)
cat("Full results saved to:", excel_onefile, "\n")

# --- Quick plots (saved as PNGs) ---
dir_plots <- file.path(out_dir, "best", "plots")
if (!dir.exists(dir_plots)) dir.create(dir_plots, recursive = TRUE)

W_long <- best_res$posterior %>%
  pivot_longer(-Year, names_to = "Sector", values_to = "Weight")
p_heat <- ggplot(W_long, aes(Year, Sector, fill = Weight)) +
  geom_tile() + scale_fill_viridis_c() +
  labs(title = "Posterior weights (W): heatmap", x = "Year", y = "Sector", fill = "Share") +
  theme_minimal(base_size = 11) + theme(axis.text.y = element_text(size = 6))
ggsave(file.path(dir_plots, "posterior_heatmap.png"), p_heat, width = 12, height = 9, dpi = 220)

top_sectors <- best_res$posterior %>%
  summarise(across(-Year, mean)) %>%
  pivot_longer(everything(), names_to = "Sector", values_to = "MeanShare") %>%
  arrange(desc(MeanShare)) %>% slice(1:3) %>% pull(Sector)  # Reduced to top 3 for demo

p_lines <- best_res$posterior %>%
  select(Year, all_of(top_sectors)) %>%
  pivot_longer(-Year, names_to = "Sector", values_to = "Weight") %>%
  ggplot(aes(Year, Weight, color = Sector)) +
  geom_line(linewidth = 0.9) +
  labs(title = "Top 3 sectors by average share (posterior W)", y = "Share", x = "Year") +
  theme_minimal(base_size = 11)
ggsave(file.path(dir_plots, "posterior_topSectors.png"), p_lines, width = 11, height = 6, dpi = 220)

cat("Demo completed successfully. All files written to temporary directory.\n")
```

# 9. Practical Guidance and Defaults

* Prefer `method="adaptive"` when prior sector volatilities are heterogeneous; otherwise `weighted` with $\lambda\in[0.7,0.9]$ is strong and often tops the grid.
* The default **coherence** parameters `(mult=3.0, const=0.5)` produce a bounded, interpretable 0–1 score that emphasizes **improvement** over the prior.
* The **exponential** numerical penalty is intentionally sharp: it keeps row-sum deviations and negatives at bay in automated runs and grid searches.
* For reports, export **Sector\_CPI** to illustrate the economic decomposition $\hat Y_{t,k}$.

# Appendix A. Invariants and Quick Checks

```{r invariants, include=TRUE}
# Example: invariants on a fresh synthetic run
T <- 6; K <- 5
set.seed(7)
P <- matrix(rexp(T*K), nrow = T); P <- P / rowSums(P)
L <- compute_L_from_P(P)
LT <- spread_likelihood(L, T, "recent")
W  <- posterior_multiplicative(P, LT)

# Invariants
stopifnot(all(abs(rowSums(P)  - 1) < 1e-12))
stopifnot(all(abs(rowSums(LT) - 1) < 1e-12))
stopifnot(all(abs(rowSums(W)  - 1) < 1e-12))
c(
  coherence = coherence_score(P, W, L),
  stability = stability_composite(W),
  interpret = interpretability_score(P, W)
) %>% round(4)
```

# Appendix B. Session Info

```{r session, include=TRUE}
sessionInfo()
```

**Notes**

- The *real-data* chunk is set to `eval=FALSE` so the manual renders anywhere. Flip it to `TRUE` on your machine to run fully against your Excel files.

- The “best one-file” export includes **Hyperparameters, Metrics, Prior_P, Posterior_W, Likelihood_t, Likelihood_L, Sector_Summary, Sector_CPI**, with frozen headers and filters for quick analysis.

- Plots are written to `.../best/plots/` and match the interpretation guidance in Section 6.

---
title: "Marco de Análisis de Sensibilidad para la Desagregación Económica Bayesiana"
author: "José Mauricio Gómez Julián"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Marco de Análisis de Sensibilidad}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

 > **Cómo leer este manual.**  
> Las secciones 1–3 desarrollan la **teoría** (con ecuaciones); las secciones 4–6 presentan **diagnósticos** y **métricas**; las secciones 7–8 proveen **código reproducible**: una demo sintética rápida (se evalúa al compilar con *knit*) y un **pipeline con datos reales** completo (deshabilitado por defecto por velocidad; habilítalo fijando `eval=TRUE`). Todo el código es consistente con las funciones exportadas por el paquete `BayesianDisaggregation`.

```{r setup, include=TRUE}
# Valores predeterminados globales de los chunks
knitr::opts_chunk$set(
  echo = TRUE, message = FALSE, warning = FALSE,
  fig.width = 9, fig.height = 6
)

# Librerías
suppressPackageStartupMessages({
  library(BayesianDisaggregation)
  library(dplyr)
  library(tidyr)
  library(ggplot2)
  library(readr)
  library(openxlsx)
})

# Verbosidad del registro desde el paquete
log_enable("INFO")
set.seed(2024)
```

# 1. Planteamiento del problema

Observamos un **índice agregado** (p. ej., IPC) por período $t=1,\dots,T$, y queremos una **desagregación sectorial** en $K$ componentes cuyas participaciones por período yacen en el **símplice unitario**:

$$
W_t = (w_{t1},\dots,w_{tK}),\qquad w_{tk}\ge 0,\quad \sum_{k=1}^K w_{tk}=1.
$$

Partimos de una **matriz de pesos previa** $P\in\mathbb{R}^{T\times K}$ (filas en el símplice), y construimos una **verosimilitud de sectores** $L\in\Delta^{K-1}$ (un vector no negativo que suma uno). Un **perfil temporal** entonces distribuye $L$ a $LT\in\mathbb{R}^{T\times K}$. Finalmente, una **regla de actualización determinista** combina $P$ y $LT$ para obtener el posterior $W$.

# 2. Construcción de la verosimilitud sectorial $L$

## 2.1 ACP/SVD de la matriz previa centrada

Sea $P$ validada (finita, no negativa, filas $\approx 1$; pequeñas desviaciones se renormalizan). **Centramos** columnas en el tiempo:

$$
X = P - \mathbf{1}\,\bar p^\top,\quad \bar p = \frac{1}{T}\sum_{t=1}^T P_{t\cdot}.
$$

Calculamos la SVD $X = U\Sigma V^\top$. Sea $v$ el **primer vector singular derecho** (cargas de la CP1). Mapeamos a una saliencia no negativa vía valores absolutos y normalizamos:

$$
\ell_k = |v_k|,\qquad L_k = \frac{\ell_k}{\sum_j \ell_j}.
$$

Si la CP1 es **degenerada** (varianza casi nula o columnas idénticas), recurrimos a las **medias de columna** de $P$ (renormalizadas). Esto está implementado en:

```{r L_from_P, include=TRUE}
# Llamada de ejemplo (los internos están en el paquete):
# L <- compute_L_from_P(P) 
```

**Diagnósticos adjuntos a `L`:** atributos `"pc1_loadings"`, `"explained_var"` y `"fallback"`.

## 2.2 Difusión temporal de $L$

Creamos una matriz dependiente del tiempo $LT$ aplicando un perfil de pesos no negativo $w_t$ y renormalizando por filas:

$$
LT_{t,k} \propto w_t L_k,\qquad \sum_k LT_{t,k}=1.
$$

**Patrones** incorporados:

* `constant`: $w_t=1$
* `recent`: linealmente creciente en $t$ (más peso a los períodos recientes)
* `linear`: rampa afín entre extremos
* `bell`: bulto simétrico tipo gaussiano alrededor de $T/2$

```{r spread-L, include=TRUE}
# Llamada de ejemplo:
# LT <- spread_likelihood(L, T_periods = nrow(P), pattern = "recent")
```

# 3. Reglas de actualización posterior (deterministas, sin MCMC)

Dados $P$ y $LT$ (ambos por filas en el símplice), definimos cuatro actualizaciones deterministas:

* **Promedio ponderado** (parámetro de mezcla $\lambda\in[0,1]$):

$$
W = \mathsf{norm}_1\{\lambda P + (1-\lambda)LT\}.
$$

* **Multiplicativa** (producto elemento a elemento con renormalización):

$$
W = \mathsf{norm}_1\{P\odot LT\}.
$$

* **Media Dirichlet** (conjugación analítica, $\gamma>0$, menor $\gamma\Rightarrow$ más aguda):

$$
\alpha_{\text{post}} = \frac{P}{\gamma} + \frac{LT}{\gamma},\qquad
W = \frac{\alpha_{\text{post}}}{\mathbf{1}^\top\alpha_{\text{post}}}.
$$

* **Adaptativa** (mezcla por sector según la volatilidad previa):

$$
\phi_k=\min\!\Big(\frac{\sigma_k}{\bar\sigma},\,0.8\Big),\quad
W_{t\cdot}=\mathsf{norm}_1\{(1-\phi)\odot P_{t\cdot} + \phi\odot LT_{t\cdot}\}.
$$

Todas están expuestas en el paquete:

```{r posteriors, include=TRUE}
# posterior_weighted(P, LT, lambda = 0.7)
# posterior_multiplicative(P, LT)
# posterior_dirichlet(P, LT, gamma = 0.1)
# posterior_adaptive(P, LT)
```

# 4. Coherencia, estabilidad e interpretabilidad

## 4.1 Coherencia respecto de $L$

Definimos las **medias temporales** previa/posterior:

$$
\bar p = \frac{1}{T}\sum_t P_{t\cdot},\qquad \bar w = \frac{1}{T}\sum_t W_{t\cdot}.
$$

Sea $\rho(\cdot,\cdot)$ una **correlación robusta** (máximo de |Pearson| y |Spearman|). La **coherencia** escala el **incremento** $\Delta\rho=\max(0,\rho(\bar w,L)-\rho(\bar p,L))$:

$$
\text{coherence}=\min\{1,\ \text{const} + \text{mult}\cdot\Delta\rho\}.
$$

## 4.2 Estabilidad numérica y temporal

* **Estabilidad numérica (penalización exponencial)** sobre desviación de suma por fila y negativos:

$$
S_{\text{num}}=\exp\{-a\cdot\overline{|\sum_k W_{tk}-1|} - b\cdot \#(W<0)\}.
$$

* **Estabilidad temporal** vía $|\Delta|$ promedio (menor variación $\Rightarrow$ mayor puntaje):

$$
S_{\text{tmp}} = \frac{1}{1+\kappa\cdot \overline{|\Delta W|}},\quad
\overline{|\Delta W|}=\frac{1}{K}\sum_k \frac{1}{T-1}\sum_{t}|W_{t+1,k}-W_{t,k}|.
$$

* **Estabilidad compuesta** (pesos predeterminados 60% numérica, 40% temporal):

$$
S_{\text{comp}} = 0.6\,S_{\text{num}} + 0.4\,S_{\text{tmp}}.
$$

Funciones del paquete:

```{r metrics-fns, include=TRUE}
# coherence_score(P, W, L, mult = 3.0, const = 0.5)
# numerical_stability_exp(W, a = 1000, b = 10)
# temporal_stability(W, kappa = 50)
# stability_composite(W, a = 1000, b = 10, kappa = 50)
```

## 4.3 Interpretabilidad

Dos principios:

1. **Preservación** de la estructura sectorial (correlación entre $\bar p$ y $\bar w$);
2. **Plausibilidad** de cambios sectoriales promedio (penalizar desplazamientos relativos extremos).

Implementación:

$$
\text{pres}=\max\{0,\rho(\bar p,\bar w)\},\qquad
r_k=\frac{|\,\bar w_k-\bar p_k\,|}{\bar p_k+\epsilon},\quad
\text{plaus}= \frac{1}{1+2\cdot Q_{0.9}(r_k)}.
$$

Luego $\text{interp}=0.6\,\text{pres}+0.4\,\text{plaus}$.

```{r interp-fn, include=TRUE}
# interpretability_score(P, W, use_q90 = TRUE)
```

# 5. API de punta a punta (`bayesian_disaggregate`)

El pipeline de conveniencia:

1. `read_cpi()` y `read_weights_matrix()` (Excel)
2. `compute_L_from_P(P)` y `spread_likelihood(L, T, pattern)`
3. regla posterior (`weighted` / `multiplicative` / `dirichlet` / `adaptive`)
4. métricas: coherencia, estabilidad (compuesta), interpretabilidad, eficiencia (heurística), puntaje compuesto
5. *helpers* de exportación: `save_results()` y un libro de Excel de un solo archivo para la configuración “mejor”

```{r api, include=TRUE}
# Firma de ejemplo (ver Sección 8 para datos reales):
# bayesian_disaggregate(path_cpi, path_weights,
#   method = c("weighted","multiplicative","dirichlet","adaptive"),
#   lambda = 0.7, gamma = 0.1,
#   coh_mult = 3.0, coh_const = 0.5,
#   stab_a = 1000, stab_b = 10, stab_kappa = 50,
#   likelihood_pattern = "recent")
```

# 6. Interpretación de visualizaciones clave

* **Mapa de calor del posterior $W$**: cada **celda** es la participación de un sector en un año; las **filas** son años, las **columnas** son sectores.
  *Léelo así*: celdas más oscuras = mayor participación sectorial; la **suavidad horizontal** indica estabilidad temporal; los **patrones verticales** (bandas) muestran importancia sectorial persistente.

* **Líneas de sectores principales**: para los sectores más relevantes por participación media, **líneas** que siguen la participación del sector en el tiempo. *Léelo así*: niveles consistentes = estabilidad; cambios de tendencia coinciden con cambios en la estructura macro.

* **Hoja de IPC sectorial**: $\hat Y_{t,k} = \text{CPI}_t \times W_{t,k}$.
  *Léelo así*: descomposición dolarizada (o escalada en índice) del agregado.

# 7. Demo sintética reproducible (se evalúa al compilar)

Este *chunk* sintetiza un ejemplo pequeño que puedes compilar con seguridad.

```{r demo, include=TRUE}
# Matriz previa sintética (filas en el símplice)
T <- 10; K <- 6
set.seed(123)
P <- matrix(rexp(T*K), nrow = T)
P <- P / rowSums(P)

# Vector de verosimilitud desde P (ACP/SVD; robusto con alternativa)
L  <- compute_L_from_P(P)

# Difundir en el tiempo con patrón "recent"
LT <- spread_likelihood(L, T_periods = T, pattern = "recent")

# Probar un par de posteriores
W_weighted <- posterior_weighted(P, LT, lambda = 0.7)
W_adaptive <- posterior_adaptive(P, LT)

# Métricas para el adaptativo
coh  <- coherence_score(P, W_adaptive, L)
stab <- stability_composite(W_adaptive, a = 1000, b = 10, kappa = 50)
intr <- interpretability_score(P, W_adaptive)
eff  <- 0.65
comp <- 0.30*coh + 0.25*stab + 0.25*intr + 0.20*eff

data.frame(coherence = coh, stability = stab, interpretability = intr,
           efficiency = eff, composite = comp) %>% round(4)
```

# 8. Pipeline completo con datos reales (habilitar/deshabilitar evaluación)

> **Cambia a `eval=TRUE` tras fijar tus rutas**. Por defecto mantenemos este *chunk* apagado para renderizar rápido en cualquier máquina.

```{r real-pipeline, eval=FALSE}
# === Crear datos sintéticos para demo compatible con CRAN ===
demo_dir <- tempdir()

# Crear datos sintéticos de IPC (imitando tu estructura)
set.seed(123)
cpi_demo <- data.frame(
  Year = 2000:2010,
  CPI = cumsum(c(100, rnorm(10, 0.5, 2)))  # Caminata aleatoria iniciando en 100
)
cpi_file <- file.path(demo_dir, "synthetic_cpi.xlsx")
openxlsx::write.xlsx(cpi_demo, cpi_file)

# Crear matriz de pesos sintética (imitando estructura de pesos de VAB)
set.seed(456)
years <- 2000:2010
sectors <- c("Agriculture", "Manufacturing", "Services", "Construction", "Mining")

weights_demo <- data.frame(Year = years)
for(sector in sectors) {
  weights_demo[[sector]] <- runif(length(years), 0.05, 0.35)
}
# Normalizar filas para sumar 1 (restricción de símplice)
weights_demo[, -1] <- weights_demo[, -1] / rowSums(weights_demo[, -1])
weights_file <- file.path(demo_dir, "synthetic_weights.xlsx")
openxlsx::write.xlsx(weights_demo, weights_file)

# Usar rutas de datos sintéticos
path_cpi <- cpi_file
path_w <- weights_file
out_dir <- demo_dir

cat("Usando datos sintéticos para la demo de CRAN:\n")
cat("Archivo CPI:", path_cpi, "\n")
cat("Archivo de pesos:", path_w, "\n")
cat("Directorio de salida:", out_dir, "\n")

# --- Ejecución base (predeterminados robustos) ---
base_res <- bayesian_disaggregate(
  path_cpi           = path_cpi,
  path_weights       = path_w,
  method             = "adaptive",
  lambda             = 0.7,   # registrado en métricas; no usado por "adaptive"
  gamma              = 0.1,
  coh_mult           = 3.0,
  coh_const          = 0.5,
  stab_a             = 1000,
  stab_b             = 10,
  stab_kappa         = 60,
  likelihood_pattern = "recent"
)
xlsx_base <- save_results(base_res, out_dir = file.path(out_dir, "base"))
print(base_res$metrics)

# --- Búsqueda de cuadrícula mínima para la demo (tamaño reducido) ---
n_cores <- 1  # Un solo núcleo para cumplimiento CRAN
grid_df <- expand.grid(
  method             = c("weighted", "adaptive"),  # Métodos reducidos
  lambda             = c(0.5, 0.7),               # Opciones reducidas
  gamma              = 0.1,                       # Opción única
  coh_mult           = 3.0,                       # Opción única  
  coh_const          = 0.5,                       # Opción única
  stab_a             = 1000,
  stab_b             = 10,
  stab_kappa         = 60,                        # Opción única
  likelihood_pattern = "recent",                  # Opción única
  KEEP.OUT.ATTRS     = FALSE,
  stringsAsFactors   = FALSE
)

grid_res <- run_grid_search(
  path_cpi     = path_cpi,
  path_weights = path_w,
  grid_df      = grid_df,
  n_cores      = n_cores
)
write.csv(grid_res, file.path(out_dir, "grid_results.csv"), row.names = FALSE)

best_row <- grid_res %>% arrange(desc(composite)) %>% slice(1)
print("Mejor configuración de la búsqueda en cuadrícula:")
print(best_row)

# --- Re-ejecutar la mejor configuración para exportación limpia ---
best_res <- bayesian_disaggregate(
  path_cpi           = path_cpi,
  path_weights       = path_w,
  method             = best_row$method,
  lambda             = if (!is.na(best_row$lambda)) best_row$lambda else 0.7,
  gamma              = if (!is.na(best_row$gamma))  best_row$gamma  else 0.1,
  coh_mult           = best_row$coh_mult,
  coh_const          = best_row$coh_const,
  stab_a             = best_row$stab_a,
  stab_b             = best_row$stab_b,
  stab_kappa         = best_row$stab_kappa,
  likelihood_pattern = best_row$likelihood_pattern
)
xlsx_best <- save_results(best_res, out_dir = file.path(out_dir, "best"))

# --- Un Excel con todo (incluyendo hiperparámetros) ---
sector_summary <- tibble(
  Sector          = colnames(best_res$posterior)[-1],
  prior_mean      = colMeans(as.matrix(best_res$prior[, -1])),
  posterior_mean  = colMeans(as.matrix(best_res$posterior[, -1]))
)

wb <- createWorkbook()
addWorksheet(wb, "Hyperparameters"); writeData(wb, "Hyperparameters", best_row)
addWorksheet(wb, "Metrics");         writeData(wb, "Metrics", best_res$metrics)
addWorksheet(wb, "Prior_P");         writeData(wb, "Prior_P", best_res$prior)
addWorksheet(wb, "Posterior_W");     writeData(wb, "Posterior_W", best_res$posterior)
addWorksheet(wb, "Likelihood_t");    writeData(wb, "Likelihood_t", best_res$likelihood_t)
addWorksheet(wb, "Likelihood_L");    writeData(wb, "Likelihood_L", best_res$likelihood)
addWorksheet(wb, "Sector_Summary");  writeData(wb, "Sector_Summary", sector_summary)

for (sh in c("Hyperparameters","Metrics","Prior_P","Posterior_W",
             "Likelihood_t","Likelihood_L","Sector_Summary")) {
  freezePane(wb, sh, firstRow = TRUE)
  addFilter(wb, sh, rows = 1, cols = 1:ncol(readWorkbook(wb, sh)))
  setColWidths(wb, sh, cols = 1:200, widths = "auto")
}

# --- Añadir IPC sectorial (agregado por pesos posteriores) ---
W_post <- best_res$posterior           # Year + sectores
cpi_df <- read_cpi(path_cpi)           # Year, CPI
sector_cpi <- dplyr::left_join(W_post, cpi_df, by = "Year") %>%
  dplyr::mutate(dplyr::across(-c(Year, CPI), ~ .x * CPI))

# Verificación de calidad: suma de sectores vs CPI
check_sum <- sector_cpi %>%
  dplyr::mutate(row_sum = rowSums(dplyr::across(-c(Year, CPI))),
                diff    = CPI - row_sum)
cat("Verificación de calidad (primeras 5 filas):\n")
print(head(check_sum, 5))

addWorksheet(wb, "Sector_CPI")
writeData(wb, "Sector_CPI", sector_cpi)
freezePane(wb, "Sector_CPI", firstRow = TRUE)
addFilter(wb, "Sector_CPI", rows = 1, cols = 1:ncol(sector_cpi))
setColWidths(wb, "Sector_CPI", cols = 1:200, widths = "auto")

excel_onefile <- file.path(out_dir, "best", "Best_Full_Output_withSectorCPI.xlsx")
saveWorkbook(wb, excel_onefile, overwrite = TRUE)
cat("Resultados completos guardados en:", excel_onefile, "\n")

# --- Gráficos rápidos (guardados como PNGs) ---
dir_plots <- file.path(out_dir, "best", "plots")
if (!dir.exists(dir_plots)) dir.create(dir_plots, recursive = TRUE)

W_long <- best_res$posterior %>%
  pivot_longer(-Year, names_to = "Sector", values_to = "Weight")
p_heat <- ggplot(W_long, aes(Year, Sector, fill = Weight)) +
  geom_tile() + scale_fill_viridis_c() +
  labs(title = "Pesos posteriores (W): mapa de calor", x = "Año", y = "Sector", fill = "Participación") +
  theme_minimal(base_size = 11) + theme(axis.text.y = element_text(size = 6))
ggsave(file.path(dir_plots, "posterior_heatmap.png"), p_heat, width = 12, height = 9, dpi = 220)

top_sectors <- best_res$posterior %>%
  summarise(across(-Year, mean)) %>%
  pivot_longer(everything(), names_to = "Sector", values_to = "MeanShare") %>%
  arrange(desc(MeanShare)) %>% slice(1:3) %>% pull(Sector)  # Reducido a top 3 para la demo

p_lines <- best_res$posterior %>%
  select(Year, all_of(top_sectors)) %>%
  pivot_longer(-Year, names_to = "Sector", values_to = "Weight") %>%
  ggplot(aes(Year, Weight, color = Sector)) +
  geom_line(linewidth = 0.9) +
  labs(title = "Top 3 sectores por participación media (posterior W)", y = "Participación", x = "Año") +
  theme_minimal(base_size = 11)
ggsave(file.path(dir_plots, "posterior_topSectors.png"), p_lines, width = 11, height = 6, dpi = 220)

cat("Demo completada exitosamente. Todos los archivos escritos al directorio temporal.\n")
```

# 9. Guía práctica y valores predeterminados

* Prefiere `method="adaptive"` cuando las volatilidades sectoriales previas son heterogéneas; de lo contrario, `weighted` con $\lambda\in[0.7,0.9]$ es sólida y suele encabezar la cuadrícula.
* Los parámetros predeterminados de **coherencia** `(mult=3.0, const=0.5)` producen un puntaje acotado e interpretable 0–1 que enfatiza la **mejora** sobre la previa.
* La penalización **exponencial** numérica es intencionalmente pronunciada: mantiene a raya las desviaciones de suma por fila y los negativos en corridas automatizadas y búsquedas en cuadrícula.
* Para informes, exporta **Sector_CPI** para ilustrar la descomposición económica $\hat Y_{t,k}$.

# Apéndice A. Invariantes y comprobaciones rápidas

```{r invariants, include=TRUE}
# Ejemplo: invariantes en una corrida sintética fresca
T <- 6; K <- 5
set.seed(7)
P <- matrix(rexp(T*K), nrow = T); P <- P / rowSums(P)
L <- compute_L_from_P(P)
LT <- spread_likelihood(L, T, "recent")
W  <- posterior_multiplicative(P, LT)

# Invariantes
stopifnot(all(abs(rowSums(P)  - 1) < 1e-12))
stopifnot(all(abs(rowSums(LT) - 1) < 1e-12))
stopifnot(all(abs(rowSums(W)  - 1) < 1e-12))
c(
  coherence = coherence_score(P, W, L),
  stability = stability_composite(W),
  interpret = interpretability_score(P, W)
) %>% round(4)
```

# Apéndice B. Información de la sesión

```{r session, include=TRUE}
sessionInfo()
```

**Notas**

- El *chunk* de *datos reales* está con `eval=FALSE` para que el manual se renderice en cualquier lugar. Cámbialo a `TRUE` en tu máquina para ejecutarlo completamente contra tus archivos de Excel.

- La exportación “de un solo archivo, la mejor” incluye **Hyperparameters, Metrics, Prior_P, Posterior_W, Likelihood_t, Likelihood_L, Sector_Summary, Sector_CPI**, con encabezados congelados y filtros para análisis rápido.

- Los gráficos se escriben en `.../best/plots/` y coinciden con la guía de interpretación de la Sección 6.

---
title: "Evidence-based Bayesian disaggregation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Evidence-based Bayesian disaggregation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## What this package does

Given an observed **aggregate** price index $\mathrm{cpi}_t$ and a matrix of
(known) sectoral aggregation weights $W_{t,k}$ — value-added (VAB) shares — the
goal is to recover the $K$ latent **sectoral** price indices $\varphi_{t,k}$ that
the aggregate is made of. The sectoral indices then feed a downstream nested
Ornstein–Uhlenbeck model (`bayesianOU`) as the market price $\varphi$.

The disaggregation is genuinely Bayesian: the aggregate enters as **evidence**
(an observation density), and the sectoral indices come out as a **posterior**
with credible intervals, not as a single deterministic re-weighting.

## Why "evidence-based": the F1–F6 history

The 0.1.x family advertised "MCMC-free Bayesian disaggregation", but the
aggregate CPI never entered the computation (F1): the "posterior" was derived
from the prior weight matrix alone, the Dirichlet concentration cancelled on
renormalization (F2), the temporal pattern cancelled too (F3), an "efficiency"
term was a fixed constant (F4), there were no recovery tests (F5), and a
correlation helper opportunistically picked whichever of Pearson/Spearman was
larger (F6). That foundational defect — not using the data — cannot be patched
within a deterministic re-weighting; the fix *is* a model that conditions on the
aggregate. The deterministic family has been removed; two honest Bayesian engines
replace it.

## The model (state-space, "Model A")

Latent state in logs, with a random walk plus drift and partial pooling:
$$
\log \varphi_{t,k} = \log \varphi_{t-1,k} + \delta_k + \tau_k\,\eta_{t,k},
\qquad \eta_{t,k}\sim\mathcal N(0,1),
$$
with $\delta_k \sim \mathcal N(\delta_\mu,\delta_\sigma)$ and
$\log\tau_k \sim \mathcal N(\mu_{\log\tau}, \sigma_{\log\tau})$ (the drift and the
innovation scale are pooled across sectors). The cross-section at $t=1$ is anchored
at the aggregate level with an **estimable** dispersion $\omega_{\text{struct}}$
(the real concentration the old Dirichlet $\gamma$ failed to be):
$$
\log\varphi_{1,k} = \log(\text{phi1\_center}) + \omega_{\text{struct}}\,z_k .
$$
The aggregate is the genuine observation:
$$
\mathrm{cpi}_t \sim \mathrm{Student\text{-}t}\!\left(\nu,\ \textstyle\sum_k W_{t,k}\,\varphi_{t,k},\ \sigma\right),
$$
(Gaussian if `student_obs = FALSE`).

### Identification, honestly (rigour by layers)

The **aggregate** $\sum_k W\varphi$ is *strongly* identified by the observation
density. The **per-sector** split is only *weakly* identified: at each period one
linear combination of the $K$ sectors is pinned by the CPI, and the remaining
$K-1$ directions are governed by the cross-sectional prior plus temporal
smoothness. So the per-sector intervals are honestly **wide and prior-influenced**.
This is not a defect to hide — it is the correct uncertainty, and it is precisely
why we feed the *full posterior draws* (not a point estimate) to the OU by
multiple imputation: the sectoral uncertainty is propagated, not faked away.

## Two engines, one trade-off

* **Closed-form (conjugate) — `disaggregate_conjugate()`**. A linear-Gaussian
  random walk in *levels* with the same aggregate observation; its exact
  posterior is the Kalman filter + RTS smoother, with no MCMC. Joint posterior
  draws come from the Durbin–Koopman simulation smoother. This is the correct
  realization of the original "MCMC-free posterior" idea.
* **MCMC — `disaggregate_statespace()`**. The richer model above (log scale ⇒
  positivity, Student-t ⇒ robustness to aggregate outliers, hierarchical pooling),
  which is *not* conjugate and therefore needs HMC.

Both are Bayesian. Closed form buys speed and exactness at the cost of a simpler
(Gaussian, linear) model; MCMC buys richness at the cost of sampling.

## A runnable example (closed-form engine)

```{r conjugate, eval = requireNamespace("BayesianDisaggregation", quietly = TRUE)}
library(BayesianDisaggregation)

sim <- simulate_disagg(T = 30, K = 4, seed = 1)   # synthetic CPI + VAB weights
bl  <- disaggregate_conjugate(sim$cpi, sim$W, n_draws = 100, seed = 1)
bl

## the smoothed aggregate tracks the CPI tightly (aggregate is well identified)
round(cor(bl$agg_summary[, "median"], sim$cpi), 4)

## joint posterior draws: the [T, K, draws] contract consumed by the nested OU
dim(bl$phi_draws)
```

## The MCMC engine (sketch, not evaluated here)

```{r statespace, eval = FALSE}
fit <- disaggregate_statespace(sim$cpi, sim$W, chains = 4, iter = 2000, warmup = 1000)
fit$diagnostics                 # rhat_max, divergences
dim(fit$phi_draws)              # T x K x draws
str(fit$phi_summary)            # median, q2.5, q97.5 (T x K each)

## couple to the nested OU (uncertainty propagated by Rubin's rule):
## bayesianOU::fit_ou_nested_mi(phi_draws = fit$phi_draws, X = Phi_index, ...)
```

From Excel directly, reusing the bundled readers:

```{r fromfiles, eval = FALSE}
cpi_file <- system.file("extdata", "CPI.xlsx", package = "BayesianDisaggregation")
w_file   <- system.file("extdata", "WEIGHTS.xlsx", package = "BayesianDisaggregation")
fit <- disaggregate_from_files(cpi_file, w_file, chains = 2, iter = 1000)
```

## Data note: comparing index vs index

The model is about **index levels**, so the CPI must be a level series (FRED
units "Index source base", aggregation "Average" for annual data — never a
rate-of-change), re-indexed to the **same base** as the production prices it will
be compared against (e.g. 1982–1984 = 100 via the project's `convert_to_index`).
Feeding a percent-change series here is a category error: the aggregate would not
be on the same scale as $\sum_k W\varphi$.

## Coupling to the nested OU

`disaggregate_statespace()$phi_draws` (or `disaggregate_conjugate(..., n_draws =
M)$phi_draws`) is a `[T, K, M]` array — exactly the multiple-imputation input of
`bayesianOU::fit_ou_nested_mi()`. The OU refits once per imputation and combines
the analyses by Rubin's rule, so the disaggregation uncertainty becomes part of
the OU posterior.
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/from_files.R
\name{align_disagg_inputs}
\alias{align_disagg_inputs}
\title{Align CPI and VAB weights on their common years}
\usage{
align_disagg_inputs(path_cpi, path_weights)
}
\arguments{
\item{path_cpi}{Path to the CPI Excel file (see \code{\link{read_cpi}}).}

\item{path_weights}{Path to the VAB-weights Excel file
(see \code{\link{read_weights_matrix}}).}
}
\value{
A list with \code{cpi} (numeric, length \eqn{T}), \code{W}
  (\eqn{T \times K}, rows sum to 1), \code{years}, \code{industries}.
}
\description{
Reads the CPI and the weights matrix, intersects their years, and returns the
aligned aggregate vector and weight matrix ready for the disaggregation
engines. Both engines (state-space and conjugate) consume this output.
}
\seealso{
\code{\link{disaggregate_from_files}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_io.R
\name{read_cpi}
\alias{read_cpi}
\title{Read CPI data from an Excel file}
\usage{
read_cpi(path_cpi)
}
\arguments{
\item{path_cpi}{Character path to the CPI Excel file.}
}
\value{
A \code{data.frame} with two columns:
\itemize{
  \item \code{Year} (\code{integer})
  \item \code{CPI}  (\code{numeric})
}
}
\description{
Loads and normalizes a CPI time series from an Excel worksheet. The function
detects the date/year column and the CPI/value column by pattern-matching on
lower-cased header names, parses localized numerics (via \code{to_num_commas()}),
collapses duplicate years by averaging, and returns a clean, sorted data frame.
}
\details{
\strong{Column detection.} Headers are lower-cased and matched with:
\itemize{
  \item Date/year: patterns \code{"date|fecha|year|anio|ano"}.
  \item CPI/value: patterns \code{"cpi|indice|price"}.
}
If either column cannot be identified, the function errors.

\strong{Cleaning.}
\itemize{
  \item Year is extracted as the first 4 digits of the date-like column.
  \item CPI is parsed with \code{to_num_commas()} (handles commas/thousands).
  \item \code{NA} rows are dropped; duplicates in \code{Year} are averaged.
  \item Output is sorted by \code{Year} ascending.
}
}
\examples{
cpi_file <- system.file("extdata", "CPI.xlsx", package = "BayesianDisaggregation")
if (nzchar(cpi_file)) {
  df <- read_cpi(cpi_file)
  head(df)
}

}
\seealso{
\code{\link{read_weights_matrix}}, \code{\link{align_disagg_inputs}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_disagg.R
\name{simulate_disagg}
\alias{simulate_disagg}
\title{Simulate from the state-space disaggregation DGP}
\usage{
simulate_disagg(
  T = 40L,
  K = 5L,
  phi1_center = 100,
  omega_struct = 0.3,
  delta_mu = 0.02,
  delta_sigma = 0.01,
  tau_mu = 0.04,
  tau_sigma = 0.3,
  sigma_cpi = 1,
  nu = Inf,
  seed = 1234L
)
}
\arguments{
\item{T}{Integer; number of periods.}

\item{K}{Integer; number of sectors.}

\item{phi1_center}{Numeric; central level of the initial cross-section.}

\item{omega_struct}{Numeric; cross-sectional log-level dispersion at t = 1.}

\item{delta_mu, delta_sigma}{Common drift and its cross-sector dispersion.}

\item{tau_mu, tau_sigma}{Geometric mean innovation scale and log-dispersion
(so \eqn{\tau_k = \tau_{mu}\exp(\tau_{sigma} z_k)}).}

\item{sigma_cpi}{Observation noise scale on the aggregate.}

\item{nu}{Student-t degrees of freedom of the observation (\code{Inf} = Gaussian).}

\item{seed}{Integer RNG seed.}
}
\value{
A list with \code{cpi} (length \eqn{T}), \code{W} (\eqn{T \times K},
  rows sum to 1), \code{phi_true} (\eqn{T \times K}), \code{agg_true}
  (length \eqn{T}), and \code{params} (the true scalar/vector parameters).
}
\description{
Generates a synthetic aggregate index \code{cpi}, the (known) VAB weights
\code{W}, and the latent sectoral price-index paths \code{phi_true} from the
same data-generating process as \code{\link{disaggregate_statespace}}. The
innovation scale is kept modest so the log random walk stays in a numerically
stable region over the simulated horizon (the same care taken in the sibling
OU simulator).
}
\examples{
sim <- simulate_disagg(T = 20, K = 3, seed = 42)
str(sim$params)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/baseline_conjugate.R
\name{disaggregate_conjugate}
\alias{disaggregate_conjugate}
\title{Conjugate (closed-form) disaggregation baseline}
\usage{
disaggregate_conjugate(
  cpi,
  W,
  years = NULL,
  industries = NULL,
  q_frac = 0.1,
  r_frac = 0.05,
  p0_frac = 0.3,
  n_draws = 0L,
  seed = 1234L
)
}
\arguments{
\item{cpi}{Numeric vector (length \eqn{T}); observed aggregate index (levels).}

\item{W}{Numeric matrix (\eqn{T \times K}); VAB weights (rows sum to 1).}

\item{years, industries}{Optional period and sector labels.}

\item{q_frac}{Random-walk innovation sd as a fraction of \code{sd(cpi)}
(state noise). Default \code{0.10}.}

\item{r_frac}{Observation sd as a fraction of \code{sd(cpi)}. Default \code{0.05}.}

\item{p0_frac}{Initial cross-sectional sd as a fraction of \code{cpi[1]}.
Default \code{0.30}.}

\item{n_draws}{Integer; number of joint posterior draws (simulation smoother).
\code{0} (default) returns only the smoothed mean and bands.}

\item{seed}{Integer RNG seed (used only when \code{n_draws > 0}).}
}
\value{
An object of class \code{"disagg_conjugate"}: a list with
  \code{phi_summary} (median = smoothed mean, \code{q2.5}/\code{q97.5} from the
  marginal Gaussian), \code{agg_summary}, \code{loglik} (total Gaussian
  log-likelihood of \code{cpi}), \code{phi_draws} (\code{[T, K, n_draws]} or
  \code{NULL}), \code{years}, \code{industries}, \code{config}.
}
\description{
Exact linear-Gaussian state-space posterior (Kalman/RTS smoother) for the
sectoral price-index levels \eqn{\varphi_{t,k}} given the aggregate index and
the VAB weights. Optionally returns joint posterior draws via the
Durbin-Koopman simulation smoother (so the draws can also feed
\code{bayesianOU::fit_ou_nested_mi}), and a pointwise Gaussian log-likelihood.
}
\examples{
sim <- simulate_disagg(T = 25, K = 4, seed = 7)
bl  <- disaggregate_conjugate(sim$cpi, sim$W)
dim(bl$phi_summary$median)
}
\seealso{
\code{\link{disaggregate_statespace}} (canonical engine).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{log_enable}
\alias{log_enable}
\title{Enable logging at a specific level}
\usage{
log_enable(level = "INFO")
}
\arguments{
\item{level}{Character scalar. One of "TRACE", "DEBUG", "INFO", "WARN", "ERROR".}
}
\value{
(Invisibly) the level set.
}
\description{
Sets the package-wide logging verbosity.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{log_msg}
\alias{log_msg}
\title{Log message with timestamp}
\usage{
log_msg(level = "INFO", ...)
}
\arguments{
\item{level}{Character level: "TRACE","DEBUG","INFO","WARN","ERROR".}

\item{...}{Message components (will be concatenated with spaces).}
}
\description{
Internal helper that prints a timestamped message when the current log
level is at least \code{level}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_io.R
\name{read_weights_matrix}
\alias{read_weights_matrix}
\title{Read a weights matrix from an Excel file}
\usage{
read_weights_matrix(path_weights)
}
\arguments{
\item{path_weights}{Character path to the weights Excel file.}
}
\value{
A \code{list} with:
\describe{
  \item{\code{P}}{\eqn{T \times K} numeric matrix of prior weights (rows sum to 1).}
  \item{\code{industries}}{Character vector of sector names (length \eqn{K}).}
  \item{\code{years}}{Integer vector of years (length \eqn{T}).}
}
}
\description{
Loads a sector-by-year weight table, normalizes weights to the simplex per year,
and returns a list with the \eqn{T \times K} prior matrix \code{P}, the sector
names, and the year vector. The first column is assumed to contain sector names
(renamed to \code{Industry}); all other columns are treated as years.
}
\details{
\strong{Expected layout.} One sheet with:
\itemize{
  \item First column: sector names (any header; renamed to \code{Industry}).
  \item Remaining columns: years; the function extracts a 4-digit year from each
        header using \code{stringr::str_extract(Year, "\\\\d{4}")}.
}
Values are parsed with \code{to_num_commas()}, missing rows are dropped, and
weights are normalized within each year to sum to 1. Any absent (sector, year)
entry becomes 0 when pivoting wide. Finally, rows are re-normalized with
\code{row_norm1()} for numerical safety.

\strong{Safeguards.}
\itemize{
  \item Rows with all-missing/zero after parsing are dropped by the filters.
  \item If no valid year columns are found, the function errors.
}
}
\examples{
w_file <- system.file("extdata", "WEIGHTS.xlsx", package = "BayesianDisaggregation")
if (nzchar(w_file)) {
  w <- read_weights_matrix(w_file)
  stopifnot(is.matrix(w$P), all(abs(rowSums(w$P) - 1) < 1e-8))
  str(w)
}

}
\seealso{
\code{\link{read_cpi}}, \code{\link{align_disagg_inputs}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disaggregate_statespace.R
\name{disagg_default_priors}
\alias{disagg_default_priors}
\title{Default prior scales for the state-space disaggregation}
\usage{
disagg_default_priors(cpi)
}
\arguments{
\item{cpi}{Numeric vector; the observed aggregate index (level).}
}
\value{
Named list of prior scales (see \code{\link{disaggregate_statespace}}).
}
\description{
Weakly-informative scales on the raw index level, derived from the observed
aggregate so the model is location/scale aware. Override any field by passing
a (partial) named list to \code{disaggregate_statespace(priors = ...)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/from_files.R
\name{disaggregate_from_files}
\alias{disaggregate_from_files}
\title{Evidence-based disaggregation directly from Excel files}
\usage{
disaggregate_from_files(path_cpi, path_weights, ...)
}
\arguments{
\item{path_cpi}{Path to the CPI Excel file (index levels, re-indexed to the
same base as the production prices; see the package vignette and the data
note on \code{convert_to_index}).}

\item{path_weights}{Path to the VAB-weights Excel file.}

\item{...}{Passed to \code{\link{disaggregate_statespace}} (sampler controls,
\code{priors}, \code{student_obs}, ...).}
}
\value{
A \code{"disagg_statespace"} object.
}
\description{
Thin convenience wrapper: reads and aligns the CPI and VAB-weight files
(\code{\link{align_disagg_inputs}}) and runs the canonical state-space engine
(\code{\link{disaggregate_statespace}}).
}
\examples{
\dontrun{
cpi_file <- system.file("extdata", "CPI.xlsx", package = "BayesianDisaggregation")
w_file   <- system.file("extdata", "WEIGHTS.xlsx", package = "BayesianDisaggregation")
fit <- disaggregate_from_files(cpi_file, w_file, chains = 2, iter = 800)
}
}
\seealso{
\code{\link{disaggregate_statespace}}, \code{\link{align_disagg_inputs}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/disaggregate_statespace.R
\name{disaggregate_statespace}
\alias{disaggregate_statespace}
\title{Evidence-based Bayesian disaggregation (state-space; canonical engine)}
\usage{
disaggregate_statespace(
  cpi,
  W,
  years = NULL,
  industries = NULL,
  student_obs = TRUE,
  priors = NULL,
  chains = 4L,
  iter = 2000L,
  warmup = 1000L,
  thin = 1L,
  cores = NULL,
  adapt_delta = 0.95,
  max_treedepth = 12L,
  seed = 1234L,
  init = 0.5,
  keep_fit = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{cpi}{Numeric vector (length \eqn{T}); the observed aggregate index in
levels (e.g. CPI re-indexed to a common base). Strictly positive.}

\item{W}{Numeric matrix (\eqn{T \times K}); the (known) VAB aggregation
weights, rows summing to 1 (small deviations are renormalized). Sector
columns must align with the desired output ordering.}

\item{years}{Optional integer vector (length \eqn{T}); period labels.}

\item{industries}{Optional character vector (length \eqn{K}); sector labels.
Defaults to \code{colnames(W)} when present.}

\item{student_obs}{Logical; if \code{TRUE} (default) the observation is
Student-t (robust to aggregate outliers), otherwise Gaussian.}

\item{priors}{Optional named list overriding \code{\link{disagg_default_priors}}.}

\item{chains, iter, warmup, thin}{Sampler controls (HMC/NUTS). Defaults
\code{4 / 2000 / 1000 / 1}.}

\item{cores}{Integer; parallel chains. Default \code{min(chains, detectCores())}.}

\item{adapt_delta, max_treedepth}{NUTS tuning. Defaults \code{0.95 / 12}.}

\item{seed}{Integer RNG seed. Default \code{1234}.}

\item{init}{Sampler init; a numeric scalar is an init radius (cmdstanr) or is
translated to \code{init_r} (rstan). Default \code{0.5}.}

\item{keep_fit}{Logical; keep the raw Stan fit object in the result. Default
\code{TRUE} (needed to draw further quantities or run LOO).}

\item{verbose}{Logical; print progress. Default \code{FALSE}.}
}
\value{
An object of class \code{"disagg_statespace"}: a list with
  \describe{
    \item{phi_draws}{\code{[T, K, draws]} numeric array of posterior draws of
      \eqn{\varphi} (the multiple-imputation input for the nested OU).}
    \item{phi_summary}{List of \eqn{T \times K} matrices \code{median},
      \code{q2.5}, \code{q97.5} (credible bands per sector and period).}
    \item{agg_summary}{\eqn{T \times 3} matrix: posterior median and 95\% band
      of the fitted aggregate \eqn{\sum_k W\varphi} (against which \code{cpi}
      is the evidence).}
    \item{years, industries}{Period and sector labels.}
    \item{diagnostics}{\code{rhat_max}, \code{divergences}.}
    \item{stan_fit}{The Stan fit (if \code{keep_fit}).}
    \item{config}{Sampler/prior configuration and \code{T}, \code{K}.}
  }
}
\description{
Disaggregates an observed aggregate index (CPI) into \eqn{K} latent sectoral
price indices \eqn{\varphi_{t,k}} with a Bayesian state-space model in which
the aggregate enters as a genuine observation density (not a renormalization
identity). The model couples a random-walk-with-drift transition in
\eqn{\log\varphi} (partial pooling on the drift and the innovation scale), an
estimable cross-sectional concentration, and a Student-t (or Gaussian)
observation \eqn{cpi_t \mid \varphi \sim \mathrm{Student\text{-}t}(\nu,
\sum_k W_{t,k}\varphi_{t,k}, \sigma)}. See \code{vignette("evidence-based-disaggregation")}.
}
\details{
The returned posterior draws of \eqn{\varphi} (a \code{[T, K, draws]} array)
are exactly the multiple-imputation input consumed by
\code{bayesianOU::fit_ou_nested_mi()}, propagating the disaggregation
uncertainty into the downstream nested-OU analysis (Rubin's rule).
}
\examples{
\dontrun{
set.seed(1)
sim <- simulate_disagg(T = 30, K = 4)
fit <- disaggregate_statespace(sim$cpi, sim$W, chains = 2, iter = 800)
dim(fit$phi_draws)            # T x K x draws
}
}
\seealso{
\code{\link{disaggregate_conjugate}} (closed-form Bayesian baseline),
  \code{\link{disaggregate_from_files}}, \code{\link{simulate_disagg}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan_backend.R
\name{disagg_stan_code}
\alias{disagg_stan_code}
\title{Stan code for the canonical disaggregation model}
\usage{
disagg_stan_code()
}
\value{
Character string with the Stan model code.
}
\description{
Returns the complete Stan code of the evidence-based state-space model read
from the canonical file (single source of truth; no embedded duplicate).
}
\examples{
code <- disagg_stan_code()
cat(substr(code, 1, 200))
}
